﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AirFusion.Dandelion
{
	/// <summary>
	/// Collection of common functions/definitions.
	/// </summary>
	internal class Common
	{
		public const bool ResultTrue = true;
		public const bool ResultFalse = false;

		public const int ResultSuccess = 0;
		public const int ResultFail = -1;

		public const int CheckInternetLoopTimes = 20;
		public const int RetryLoopTimes = 3;
		public const int DefaultHoldTime = 5;
		public const int SocketTimeout = 5000;
		public const int LoginTimeout = 10000;

		/// <summary>
		/// Defines action for the AODController class.
		/// </summary>
		public enum AODControllerAction : int
		{
			/// <summary>
			/// None.
			/// </summary>
			ActionNone = 0,

			/// <summary>
			/// Registers the device ID.
			/// </summary>
			ActionRegisterDeviceID = 1,

			/// <summary>
			/// Updates the profile.
			/// </summary>
			ActionUpdateProfile = 2
		}

		/// <summary>
		/// Specifies the status code of http response.
		/// </summary>
		public enum HttpStatusCode : int
		{
			/// <summary>
			/// The request has succeeded.
			/// </summary>
			HttpStatusCodeOK = 200,

			/// <summary>
			/// The request has been fulfilled and resulted in a new resource being created.
			/// </summary>
			HttpStatusCodeCreated = 201,

			/// <summary>
			/// The server has fulfilled the request but does not need to return an entity-body,
			/// and might want to return updated metainformation.
			/// </summary>
			HttpStatusCodeNoContent = 204,

			/// <summary>
			/// The request could not be understood by the server due to malformed syntax.
			/// </summary>
			HttpStatusCodeBadRequest = 400,

			/// <summary>
			/// The request requires user authentication. The response MUST include a WWW-Authenticate
			/// header field containing a challenge applicable to the requested resource.
			/// </summary>
			HttpStatusCodeUnauthorized = 401,

			/// <summary>
			/// The requested resource could not be found but may be available again in the future. Subsequent
			/// requests by the client are permissible.
			/// </summary>
			HttpStatusCodeNotFound = 404,

			/// <summary>
			/// The server timed out waiting for the request.
			/// </summary>
			HttpStatusCodeRequestTimeout = 408,

			/// <summary>
			/// Indicates that the request could not be processed because of conflict in the request, such as
			/// an edit conflict in the case of multiple updates.
			/// </summary>
			HttpStatusCodeConflict = 409,

			/// <summary>
			/// The server encountered an unexpected condition which prevented it from fulfilling the request.
			/// </summary>
			HttpStatusCodeInternalServerError = 500,

			/// <summary>
			/// The Request-URI is null or invalid.
			/// </summary>
			HttpStatusCodeValueInvalid = 901,
		}

		#region Flag and Path for Logger

		/// <summary>
		/// Enable/Disable the logger output
		/// </summary>
		public static bool EnableOutputToLogFile = false;

		public static int LogFileLimit = 200;

		public const string Product = "AcerEXTEND";
		private const string _componentName = "AODFramework";
		private const string _acerFolder = "OEM\\" + Product;
		public static string CommAppFolderPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData), _acerFolder, _componentName);
		public static string LogFolderPath = Path.Combine(CommAppFolderPath, "log-");

		#endregion Flag and Path for Logger

		#region Constants and Fields for Registry

		public const string RegRootKeyName = "HKEY_CURRENT_USER";
		public const string RegFirstSubKeyName = "Software\\OEM\\AcerEXTEND";
		public const string RegSecondSubKeyName = "Dandelion";
		public const string RegAccessTokenKeyName = "access_token";
		public const string RegRefreshTokenKeyName = "refresh_token";
		public const string RegLoginTypeKeyName = "login_type";
		public const string RegUserIDKeyName = "id";
		public const string RegUserNameKeyName = "username";
		public const string RegDeviceIDKeyName = "device_id";
		public const string RegLogStatusKeyName = "LogStatus";

		#endregion Constants and Fields for Registry

		#region Constants and Fields for Http

		public const string HttpMethodGet = "GET";
		public const string HttpMethodPut = "PUT";
		public const string HttpMethodPost = "POST";
		public const string HttpMethodDelete = "DELETE";

		public const string HttpContentTypeJson = "application/json";
		public const string HttpContentTypeUrlencoded = "application/x-www-form-urlencoded";

		public const string HttpHeaderAuthorization = "Authorization";
		public const string HttpHeaderBearer = "Bearer ";

		public const string HttpOauthResponseTypeCode = "code";

		public const string HttpOauthGrantTypeAuthorization = "authorization_code";
		public const string HttpOauthGrantTypeRefreshToken = "refresh_token";

		public const string HttpOauthRedirectUri = "urn:ietf:wg:oauth:2.0:oob";

		#endregion Constants and Fields for Http

		#region Constants and Fields for Dandelion

		public const int PairMethodSonudTrigger = 4;
		public const int PairMethodBSSID = 8;

		public const bool EnableAPIKey = true;

		//public const string DandelionAirFusionAppKey = "api_key=2575b043c202314e77b021fa69a4266fcfaea466";
		//public const string DandelionPairAppKey = "api_key=615b30da97f81bf6b36d00f358086b340efc6811";
		public const string DandelionAirFusionAppKey = "api_key=749f699bb3c338c3e9bfb96ceab9e2b9d1eb57b1";

		public const string DandelionPairAppKey = "api_key=fe7c1e7f20e579916449acc8bd4cccb5dc11acad";

		#endregion Constants and Fields for Dandelion

		public static bool CheckLogEnableStatus()
		{
			bool isEnable = true;

			RegistryUtils tmpReg = new RegistryUtils();
			string tmpKeyPath = Common.RegRootKeyName + "\\" + Common.RegFirstSubKeyName + "\\" +
																				Common.RegSecondSubKeyName;
			int logStatus = -1;
			try
			{
				logStatus = (int)tmpReg.GetRegistryValue(tmpKeyPath, Common.RegLogStatusKeyName, -1);
			}
			catch
			{
				return false;
			}

			if (logStatus <= 0)
			{
				isEnable = false;
			}

			return isEnable;
		}
	}
}
