﻿using AirFusion.Common;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace AirFusion
{
	/// <summary>
	/// Class for fundamental configuration values
	/// </summary>
	public class Configs
	{
		public const string Product = "AcerEXTEND";            // Product name
		public const string ProductTray = "AirFusionTray";    // Product Tray Window Name
		private const string _componentName = "UIProgram";  // For this component
		private const string _acerFolder = "OEM\\" + Product;

		public static string CommAppFolderPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData), _acerFolder, _componentName);
		public static string CommAppDataPath = CommonUtils.ConvertPath(@"%APPDATA%\" + Product);

		public const string DeviceInfoFileName = "DeviceManager.ini";
		private const string _screenshotFolderName = "Screenshots";
		public static string ScreenshotFileFolderPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.MyPictures), _screenshotFolderName);
		private const string _downloadFolderName = Product;
		public static string DownloadFileFolderPath = Path.Combine(Utils.GetDownloadFolder(), _downloadFolderName);
		public static string VideoFileFolderPath = Environment.GetFolderPath(Environment.SpecialFolder.MyVideos);

		public static string LogPath = Path.Combine(CommAppFolderPath, "log");
		public const string RegPath = @"Software\OEM\" + Product;
		public const string RegVideoPath = RegPath + @"\Video";
		public const string RegVideoResolutionKey = "Resolution";
		public const string RegVideoResolutionWidthKey = "ResolutionW";
		public const string RegVideoResolutionHeightKey = "ResolutionH";
		public const string RegVideoIntervalKey = "KeyFrameInterval";
		public const string RegVideoHighBiterate = "HighBiteRate";
		public const string RegVideoLowBiterate = "LowBiteRate";
		public const string RegPhoneSystemPermissionDetectionRuleKey = "PhoneSystemPermissionDetectionRule";
		public const string RegVideoProtocolKey = "VideoProtocol";
		public const string RegAudioPath = RegPath + @"\Audio";
		public const string RegAudioDisableKey = "Disable";

		//0602 Johnson add for demo disable stop screencast timer
		public const string RegDisableStopScreencastEventKey = "StopScreencast";

		public const string RegUIPath = RegPath + @"\UI";
		public const string RegUIGeneralPath = RegUIPath + @"\General";
		public const string RegLastLaunchIDKey = "LastLaunchID";
		public const string RegUISettingsPath = RegUIPath + @"\Settings";
		public const string RegNotificationKey = "Notification";
		public const string RegUnlockKey = "Unlock";
		public const string RegLoginAccountKey = "LoginAccount";
		public const string RegLoginPWDKey = "LoginPWD";
		public const string RegAnswerPhoneCallKey = "AnswerPhoneCall";
		public const string RegMute = "Mute";
		public const string RegAutoUpdate = "AutoUpdate";

		public const string RegStartUpLaunchPath = @"Software\Microsoft\Windows\CurrentVersion\Run";
		public const string RegStartUpLaunch = "StartUpLaunch";

		public const int DefaultRTSPPort = 7236;
		public const int DefaultRTPPort = 24030;
		public const uint DefaultVNCPort = 5901;
		public const string DefaultVNCPassword = "12345678";

		public const int RetryTimesForInputControl = 3;
		public const int RetryTimePeriodForInputControl = 1000;			// with microsecond
		public const int RetryTimesForScreenCasting = 10;
		public const int RetryTimePeriodForScreenCasting = 3000;		// with microsecond
		public const int ReceiveTimeoutForRTSPSocket = 5000;			// with microsecond

		public const double WaitForPairingTimeoutPeriod = 180000;		// with microsecond

		//For Computex Test, change receive data time to 10s
		public const double WaitForReceiveDataTimeoutPeriod = 5000;	// with microsecond

		public const double RetryTimesForCheckInputDaemonStatus = 3;
		public const double RetryTimesForRunInputDaemon = 5;
		public const int RetryTimePeriodForWaitInputDaemonReady = 1000;				// with microsecond

		public const double RetryTimesForCheckScreenCastingDaemonStatus = 5;
		public const double RetryTimesForRunScreencastingDaemon = 3;
		public const int RetryTimePeriodForWaitScreencastingDaemonReady = 500;		// with microsecond
		public const double RetryTimesForPlayScreencast = 3;
		public const int RetryTimePeriodForWaitPlayScreencastingDaemonReady = 500;

		public const double OptimalWindowRatioinScreen = 0.8;

		public const short LengthOfSkipVideoFrame = 5;

		public const int DefaultVideoResolutionShortSize = 720;
		public const int DefaultVideoResolutionLongSize = 1280;

		public const bool IsWithoutMiracast = true;   // For temporary debug use

		// This App reference is for current application
		public static App ApplicationReference = (Application.Current as App);

		public static DebugLevel DebugLevelValue = DebugLevel.All;

		// These configurations are for ADB commands
		private static string _adbBinaryFileFolder = System.Windows.Forms.Application.StartupPath;

		private const string _inputDaemonFileName = "input_daemon";
		private const string _screenCastingDaemonFileName = "screencast";
		private const string _remoteDaemonOriginalFileFolder = "/sdcard/Android/data/com.acer.android.airfusion/";
		private const string _remoteDaemonTempFileFolder = "/data/local/tmp/";
		private static string _localInputDaemonFilePath = Path.Combine(CommAppDataPath, _inputDaemonFileName);

		//private static string _localInputDaemonFilePath = CommAppDataPath;
		private static string _localScreenCastingDaemonFilePath = Path.Combine(CommAppDataPath, _screenCastingDaemonFileName);

		//private static string _localScreenCastingDaemonFilePath = CommAppDataPath;
		public static string ADBProcessName = "AirFusionadb";

		public static string CheckADBFilePath = Path.Combine(_adbBinaryFileFolder, "adb.exe");
		public static string ADBFilePath = Path.Combine(_adbBinaryFileFolder, ADBProcessName + ".exe");
		public const string DeviceCommand = "devices";
		public const string KillServerCommand = "-s {0} kill-server";
		public const string DeviceExistCommand = "-s {0} shell pwd";
		public static string PullInputDaemonCommand = string.Join(" ", new string[] { "-s {0}", "pull", string.Format("\"{0}\"", _remoteDaemonOriginalFileFolder + _inputDaemonFileName), string.Format("\"{0}\"", _localInputDaemonFilePath) });
		public static string PushInputDaemonCommand = string.Join(" ", new string[] { "-s {0}", "push", string.Format("\"{0}\"", _localInputDaemonFilePath), string.Format("\"{0}\"", _remoteDaemonTempFileFolder) });
		public static string PullScreenCastingCommand = string.Join(" ", new string[] { "pull", string.Format("\"{0}\"", _remoteDaemonOriginalFileFolder + _screenCastingDaemonFileName), string.Format("\"{0}\"", _localScreenCastingDaemonFilePath) });
		public static string PushScreenCastingCommand = string.Join(" ", new string[] { "push", string.Format("\"{0}\"", _localScreenCastingDaemonFilePath), string.Format("\"{0}\"", _remoteDaemonTempFileFolder) });
		public static string ShellSetPrivilegeInputDaemonCommand = string.Join(" ", new string[] { "-s {0}", "shell chmod 755", string.Format("\"{0}\"", _remoteDaemonTempFileFolder + _inputDaemonFileName) });

		//0430 Johnson modify for Andorid request modify command
		//public static string ShellInputDaemonCommand = string.Join(" ", new string[] { "shell", string.Format("\"{0}\"", _remoteDaemonTempFileFolder + _inputDaemonFileName + " airfusion &") });
		public static string ShellInputDaemonCommand = string.Join(" ", new string[] { "shell", string.Format("\"{0}\"", _remoteDaemonTempFileFolder + _inputDaemonFileName + " 0 &") });

		public static string ShellSetPrivilegeScreenCastingDaemonCommand = string.Join(" ", new string[] { "shell chmod 755", string.Format("\"{0}\"", _remoteDaemonTempFileFolder + _screenCastingDaemonFileName) });
		public static string ShellscreenCastingDaemonCommand = string.Join(" ", new string[] { "shell", string.Format("\"{0}\"", _remoteDaemonTempFileFolder + _screenCastingDaemonFileName + " &") });
		public static string ShellCheckRunScreenCastingDaemonCommand = string.Join(" ", new string[] { "shell", string.Format("\"{0}\"", "ps | grep"), _screenCastingDaemonFileName });

		//0411 Johnson add
		public static string AcerEXTENDInstallGUID = @"{96FE4347-73D3-4929-A485-7C76EF15B1F7}";

		public static string AcerEXTENDInstallKeyX64 = string.Format("SOFTWARE\\Wow6432Node\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\{0}", AcerEXTENDInstallGUID);
		public static string AcerEXTENDInstallKeyX86 = string.Format("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\{0}", AcerEXTENDInstallGUID);

		//0608 Johnson add

		#region For WebService(Device Status) Query parameter

		public const double CheckRemoteDeviceStatusRetryTimes = 3;
		public const int CheckRemoteDeviceStatusRetryTimePeriod = 1000; //1s

		#endregion For WebService(Device Status) Query parameter

		#region For WebService(Device Info) Query parameter

		public const double CheckRemoteDeviceInfoRetryTimes = 3;
		public const int CheckRemoteDeviceInfoRetryTimePeriod = 1000; //1s

		#endregion For WebService(Device Info) Query parameter

		#region For WebService(Input Daemon Status) Query parameter

		public const double CheckRemoteInputDaemonStatusRetryTimes = 3;
		public const int CheckRemoteInputDaemonStatusRetryTimePeriod = 1000; //1s

		#endregion For WebService(Input Daemon Status) Query parameter

		#region For WebService(Screencast Daemon Status) Query parameter

		public const double CheckRemoteScreencasetDaemonStatusRetryTimes = 3;
		public const int CheckRemoteScreencasetDaemonStatusRetryTimePeriod = 1000; //1s

		#endregion For WebService(Screencast Daemon Status) Query parameter

		#region For WebService(Play Screencast) parameter

		public const double PlayRemoteSceencastRetryTimes = 3;
		public const int PlayRemoteScreencastRetryTimePeriod = 1000; //1s

		#endregion For WebService(Play Screencast) parameter

		#region For WebService(Stop Screencast) parameter

		public const double StopRemoteScreencastRetryTimes = 1;

		#endregion For WebService(Stop Screencast) parameter
	}
}