﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AirFusion.ConnectionManager
{
	#region Define event triggered from Connection flow

	/// <summary>
	/// Enumeration for event type from connection flow
	/// </summary>
	public enum ConnectionEventType
	{
		None,

		CheckRemoteDeviceStatusSuccess,
		CheckRemoteDeviceStatusFail,

		CheckRemoteControlStatusSuccess,
		CheckRemoteControlStatusFail,

		ScreencastControlSuccess,
		ScreencastControlFail,
	}

	public enum ConnectDeviceStatusType
	{
		None,

		#region Wakeup Status

		WakeupSuccess,
		WakeupFail,
		ConfirmIPFail,
		DandelionFail,
		VersionNotMatch,

		#endregion Wakeup Status

		#region Get Device Status

		CheckDeviceStatusPair,
		CheckDeviceStatusUnPair,
		CheckDeviceStatusTimeout,
		CheckDeviceStatusFail,

		#endregion Get Device Status

		#region Get Device Info

		GetDeviceInfoSuccess,
		GetDeviceInfoTimeout,
		GetDeviceInfoFail,

		#endregion Get Device Info
	}

	public enum ConnectControlStatusType
	{
		None,

		#region Connection to Remote Input Control Server status

		InputControlConnectSuccess,
		InputControlConnectFail,
		InitialInputControlFail,

		#endregion Connection to Remote Input Control Server status

		#region Check remote Input Daemon Status

		InputDaemonNotRunning,
		InputDaemonRunning,
		CheckInputDaemonTimeout,
		CheckInputDaemonWebException,
		CheckInputDaemonException,

		#endregion Check remote Input Daemon Status

		#region Check remote Screencast Daemon Status

		ScreencastDaemonNotRunning,
		ScreencastDaemonRunning,
		CheckScreencastDaemonTimeout,
		CheckScreencastDaemonWebException,
		CheckScreencastDaemonException,

		#endregion Check remote Screencast Daemon Status

		#region For Non-Acer Device Stauts Type
		ADBKillServerSuccess,
		ADBKillServerFail,//Device not exist
		//ADBRemoteDeviceError,
		ADBRemoteDeviceNotFound,
		//ADBRemoteDeviceFound,
		ADBPullInputDaemonFail,
		//ADBPullInputDaemonSuccess,
		ADBPushInputDaemonFail,
		//ADBPushInputDaemonSuccess,
		ADBShellSetPrivilegeInputDaemonFail,
		ADBShellSetPrivilegeInputDaemonSuccess,
		LaunchInputDaemonByADBException,
		LaunchInputDaemonByADBSuccess,

		ADBPullScreencastDaemonFail,
		ADBPullScreencastDaemonSuccess,
		ADBPushScreencastDaemonFail,
		ADBPushScreencastDaemonSuccess,
		ADBShellSetPrivilegeScreencastDaemonFail,
		ADBShellSetPrivilegeScreencastDaemonSuccess,
		LaunchScreencastDaemonByADBException,
		LaunchScreencastDaemonByADBSuccess,
		#endregion
	}

	public enum ScreencastStatusType
	{
		None,

		ScreencastControlSuccess,
		ScreencastControlTimeout,
		ScreencastControlWebServiceException,
		ScreencastControlException,
	}

	public enum DaemonType
	{
 		InputDaemon,
		ScreencastDaemon,
	}

	/// <summary>
	/// Class for event argument from connection flow
	/// </summary>
	public class ConnectEventArgs : EventArgs
	{
		public ConnectEventArgs(ConnectionEventType eventType, object data = null, object args = null)
		{
			this._eventType = eventType;
			this._data = data;
			this._args = args;
		}

		private ConnectionEventType _eventType;

		public ConnectionEventType EventType
		{
			get
			{
				return _eventType;
			}
		}

		private object _data = null;

		public object Data
		{
			get
			{
				return _data;
			}
		}

		private object _args = null;

		public object Args
		{
			get
			{
				return _args;
			}
		}
	}

	#endregion Define event triggered from Connection flow
}