﻿using System;

namespace AirFusion.Common.RegistryHolders
{
	/// <summary>
	/// Specifies the store type.
	/// </summary>
	public enum StoreType
	{
		/// <summary>
		/// Signifies to stores the property as a sub tree.
		/// </summary>
		SubTree,

		/// <summary>
		/// Signifies to stores the property as its value.
		/// </summary>
		Value,

		/// <summary>
		/// Signifies the property is not going to be stored.
		/// </summary>
		NoStore
	}

	/// <summary>
	/// Represents the store type attribute of a property.
	/// </summary>
	[AttributeUsage(AttributeTargets.Property, AllowMultiple = false)]
	public sealed class RegistryStoreTypeAttribute : Attribute
	{
		/// <summary>
		/// Gets or sets the type of the converter.
		/// </summary>
		/// <value>
		/// The type of the converter.
		/// </value>
		public StoreType StoreType { get; private set; }

		/// <summary>
		/// Initializes a new instance of the <see cref="RegistryStoreTypeAttribute"/> class.
		/// </summary>
		/// <param name="storeType">The store type.</param>
		public RegistryStoreTypeAttribute(StoreType storeType)
		{
			this.StoreType = storeType;
		}
	}
}