﻿namespace AirFusion.Common.Wmi.Cim
{
	/// <summary>
	/// The CIM_System class aggregates an enumerable set of managed system elements. The aggregation operates as a functional whole.
	/// Within any particular subclass of the system, there is a well-defined list of managed system element classes whose instances must be aggregated.
	/// The CIM_System object and its derivatives are top-level objects of CIM. They provide the scope for numerous components and are required to have unique system keys.
	/// </summary>
	[ParameterValue("CIM_System")]
	public class WmiSystem : WmiLogicalElement
	{
		/// <summary>
		/// Gets the name of the class or subclass used in the creation of an instance.
		/// When used with other key properties of the class, this property allows all instances of the class and its subclasses to be uniquely identified.
		/// </summary>
		[ParameterValue("CreationClassName")]
		public string CreationClassName { get; internal set; }

		/// <summary>
		/// Gets the identifies how the system name was generated, using the subclass heuristic.
		/// </summary>
		[ParameterValue("NameFormat")]
		public string NameFormat { get; internal set; }

		/// <summary>
		/// Gets the contact that represents how the primary system owner can be reached (for example, phone number or email address).
		/// </summary>
		[ParameterValue("PrimaryOwnerContact")]
		public string PrimaryOwnerContact { get; internal set; }

		/// <summary>
		/// Gets the name of the primary system owner.
		/// </summary>
		[ParameterValue("PrimaryOwnerName")]
		public string PrimaryOwnerName { get; internal set; }

		/// <summary>
		/// Gets the roles the system plays in the information technology environment.
		/// Subclasses of the system can override this property to define explicit role values.
		/// Alternately, a working group can describe the heuristics, conventions, and guidelines for specifying roles.
		/// For example, for an instance of a networking system, this property might contain the string "Switch" or "Bridge".
		/// </summary>
		[ParameterValue("Roles")]
		public string[] Roles { get; internal set; }

		/// <summary>
		/// Initializes the object.
		/// </summary>
		public WmiSystem()
		{
			this.CreationClassName = null;
			this.NameFormat = null;
			this.PrimaryOwnerContact = null;
			this.PrimaryOwnerName = null;
			this.Roles = null;
		}
	}
}