﻿using AirFusion.Common;
using AirFusion.Windows;
using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows;
using System.Windows.Forms;
using System.Windows.Input;
using System.Windows.Media.Animation;

namespace AirFusionCustomToast
{
	public partial class ToastPopUp : Window, System.ComponentModel.INotifyPropertyChanged
	{
		#region Private Members

		private readonly string typeNameofToast = typeof(ToastPopUp).Name;
		private volatile object lockToast = new object();
		private ToastUIElement _currentToast = null;
		private System.Timers.Timer timer;
		private bool isPhoneCallToast = false;

		#endregion Private Members

		#region Public Properties

		public ToastUIElement CurrentToast
		{
			get
			{
				return _currentToast;
			}
			set
			{
				_currentToast = value;
				OnPropertyChanged("CurrentToast");
			}
		}

		public delegate void PressedEventHandler(object sender, PressedEventArgs e);

		public delegate void ToastActionTriggeredEventHandler(object sender, ToastActionTriggeredEventArgs e);

		public delegate void ToastClosedEventHandler(object sender, ToastClosedEventArgs e);

		#endregion Public Properties

		#region Events

		public event PropertyChangedEventHandler PropertyChanged;

		public event PressedEventHandler PressedEvent;

		public event ToastActionTriggeredEventHandler ToastActionTriggeredEvent;

		public event ToastClosedEventHandler ToastClosedEvent;

		#endregion Events

		#region Constructor

		public ToastPopUp(ToastUIElement toastUIElement, bool _isPhoneCallToast)
			: base()
		{
			isPhoneCallToast = _isPhoneCallToast;

			this.InitializeComponent();

			try
			{
				this.CurrentToast = toastUIElement;
			}
			catch (Exception ex)
			{
				System.Windows.Forms.MessageBox.Show(ex.ToString());
			}

			if (isPhoneCallToast)
			{
				this.grdIncomingPhoneCallButtons.Visibility = Visibility.Visible;
			}
		}

		#endregion Constructor

		#region Callback Functions

		#region Event Handlers

		private void ToastClosed(object sender, EventArgs e)
		{
			foreach (Window window in System.Windows.Application.Current.Windows)
			{
				string windowName = window.GetType().Name;

				if (windowName.Equals(this.typeNameofToast) && window != this)
				{
					if (window.Top < (this.Top - 5) && window.Left == this.Left)
					{
						window.Top = window.Top + this.ActualHeight + 5;

						if (!WindowsExistToTheRight(this.Left))
						{
							window.Left = window.Left + this.ActualWidth + 5;
						}
					}
				}
			}

			this.AdjustWindows();

			this.Closed -= this.ToastClosed;
		}

		private void MouseUpCloseToast(object sender, MouseButtonEventArgs e)
		{
			if (this.ToastClosedEvent != null)
			{
				this.ToastClosedEvent(this, new ToastClosedEventArgs()
				{
					ToastPopUp = this,
				});
			}

			e.Handled = true;
		}

		private void MouseUpPressToast(object sender, MouseButtonEventArgs e)
		{
			if (this.PressedEvent != null)
			{
				this.PressedEvent(this, new PressedEventArgs()
				{
					// some data to return while pressed
				});
			}
		}

		private void DoubleAnimationCompleted(object sender, EventArgs e)
		{
			if (!this.IsMouseOver)
			{
				if (this.ToastClosedEvent != null)
				{
					this.ToastClosedEvent(this, new ToastClosedEventArgs()
					{
						ToastPopUp = this,
					});
				}
			}
		}

		private void btnAnswer_Click(object sender, RoutedEventArgs e)
		{
			this.btnAnswer.IsEnabled = false;

			if (this.ToastActionTriggeredEvent != null)
			{
				this.ToastActionTriggeredEvent(this, new ToastActionTriggeredEventArgs()
				{
					PhoneCallToastID = this.CurrentToast.NotificationKey,
					PhoneCallToastAction = PhoneCallToastActionType.Answer
				});
			}
		}

		private void btnDecline_Click(object sender, RoutedEventArgs e)
		{
			if (this.ToastActionTriggeredEvent != null)
			{
				this.ToastActionTriggeredEvent(this, new ToastActionTriggeredEventArgs()
				{
					PhoneCallToastID = this.CurrentToast.NotificationKey,
					PhoneCallToastAction = PhoneCallToastActionType.Decline
				});
			}
		}

		private void btnMessage_Click(object sender, RoutedEventArgs e)
		{
			if (this.ToastActionTriggeredEvent != null)
			{
				this.ToastActionTriggeredEvent(this, new ToastActionTriggeredEventArgs()
				{
					PhoneCallToastID = this.CurrentToast.NotificationKey,
					PhoneCallToastAction = PhoneCallToastActionType.Message
				});
			}
		}

		private void btnMute_Click(object sender, RoutedEventArgs e)
		{
			if (this.ToastActionTriggeredEvent != null)
			{
				this.ToastActionTriggeredEvent(this, new ToastActionTriggeredEventArgs()
				{
					PhoneCallToastID = this.CurrentToast.NotificationKey,
					PhoneCallToastAction = PhoneCallToastActionType.Mute
				});
			}
		}

		private void btnEnd_Click(object sender, RoutedEventArgs e)
		{
			if (this.ToastActionTriggeredEvent != null)
			{
				this.ToastActionTriggeredEvent(this, new ToastActionTriggeredEventArgs()
				{
					PhoneCallToastID = this.CurrentToast.NotificationKey,
					PhoneCallToastAction = PhoneCallToastActionType.End
				});
			}
		}

		private void ToastWindow_MouseEnter(object sender, System.Windows.Input.MouseEventArgs e)
		{
			if (!isPhoneCallToast)
			{
				Storyboard storyboardLoad = this.FindResource("StoryboardLoad") as Storyboard;
				storyboardLoad.Remove();
				Storyboard storyboardFade = this.FindResource("StoryboardFade") as Storyboard;
				storyboardFade.Remove();
			}
		}

		private void ToastWindow_MouseLeave(object sender, System.Windows.Input.MouseEventArgs e)
		{
			if (!isPhoneCallToast)
			{
				Storyboard storyboardFade = this.FindResource("StoryboardFade") as Storyboard;
				storyboardFade.Begin();
			}
		}

		private void ToastWindow_Loaded(object sender, RoutedEventArgs e)
		{
			if (!isPhoneCallToast)
			{
				Storyboard storyboardLoad = this.FindResource("StoryboardLoad") as Storyboard;
				storyboardLoad.Begin();
			}
		}

		#endregion Event Handlers

		#endregion Callback Functions

		#region Methods

		private void AdjustWindows()
		{
			lock (lockToast)
			{
				Rectangle workingArea = Screen.PrimaryScreen.WorkingArea;

				this.Left = SystemParameters.WorkArea.Width - this.ActualWidth - 10;
				double top = SystemParameters.WorkArea.Height - this.ActualHeight - 10;

				foreach (Window window in System.Windows.Application.Current.Windows)
				{
					string windowName = window.GetType().Name;

					if (windowName.Equals(this.typeNameofToast) && window != this)
					{
						window.Topmost = true;

						if (this.Left == window.Left)
						{
							top = top - window.ActualHeight - 5;
						}

						if (top < 0)
						{
							this.Left = this.Left - this.ActualWidth - 5;
							top = workingArea.Bottom - this.ActualHeight - 10;
						}
					}
				}

				this.Top = top;
			}
		}

		private bool WindowsExistToTheRight(double left)
		{
			foreach (Window window in System.Windows.Application.Current.Windows)
			{
				string windowName = window.GetType().Name;

				if (windowName.Equals(this.typeNameofToast) && window != this &&
					left == Screen.PrimaryScreen.WorkingArea.Width - this.Width - 10)
				{
					return true;
				}
			}

			return false;
		}

		public static string normalizeTalkTime(int talkTimeInSeconds)
		{
			string hour = string.Empty;
			string minute = string.Empty;
			string second = string.Empty;

			hour = (talkTimeInSeconds / 3600).ToString();
			if (hour.Length < 2)
			{
				hour = "0" + hour;
			}

			minute = (talkTimeInSeconds % 3600 / 60).ToString();
			if (minute.Length < 2)
			{
				minute = "0" + minute;
			}

			second = (talkTimeInSeconds % 60).ToString();
			if (second.Length < 2)
			{
				second = "0" + second;
			}

			return hour + ":" + minute + ":" + second;
		}

		public new void Show()
		{
			var focusedElement = Keyboard.FocusedElement;

			this.Topmost = true;
			base.Show();

			this.Owner = System.Windows.Application.Current.MainWindow;
			this.Closed += this.ToastClosed;
			this.AdjustWindows();

			if (focusedElement != null)
			{
				focusedElement.Focusable = true;
				Keyboard.Focus(focusedElement);
			}
		}

		public void RefreshToastUI(string lineState)
		{
			this.Dispatcher.BeginInvoke(new Action(() =>
			{
				int talkTime = 0;
				switch (lineState)
				{
					case "AnswerOnPC":
						{
							this.grdIncomingPhoneCallButtons.Visibility = Visibility.Collapsed;
							this.grdOntheLineButtons.Visibility = Visibility.Visible;

							timer = new System.Timers.Timer(1000);
							this.CurrentToast.ContentLine1 = string.Empty;
							timer.Elapsed += (sender, e) =>
							{
								talkTime++;
								this.CurrentToast.ContentLine1 = normalizeTalkTime(talkTime);
							};
							timer.AutoReset = true;
							timer.Enabled = true;

							break;
						}
					case "AnswerOnPhone":
						{
							this.grdIncomingPhoneCallButtons.Visibility = Visibility.Collapsed;
							this.grdOntheLineButtons.Visibility = Visibility.Visible;

							timer = new System.Timers.Timer(1000);
							this.CurrentToast.ContentLine1 = string.Empty;
							timer.Elapsed += (sender, e) =>
							{
								talkTime++;
								this.CurrentToast.ContentLine1 = normalizeTalkTime(talkTime);
							};
							timer.AutoReset = true;
							timer.Enabled = true;

							break;
						}
					case "HangUpFromPC":
						{
							try
							{
								if (timer != null) timer.Close();
							}
							catch (Exception ex)
							{
								StaticLogger.LogInfo("[###] RefreshToastUI => timer.Close(): " + ex.ToString());
							}

							try
							{
								if (this.ToastClosedEvent != null)
								{
									this.ToastClosedEvent(this, new ToastClosedEventArgs()
									{
										ToastPopUp = this,
									});
								}
							}
							catch (Exception ex)
							{
								StaticLogger.LogInfo("[###] RefreshToastUI => this.Close() " + ex.ToString());
							}

							break;
						}
					case "HangUpFromPhone":
						{
							try
							{
								if (timer != null) timer.Close();
							}
							catch (Exception ex)
							{
								StaticLogger.LogInfo("[###] RefreshToastUI => timer.Close(): " + ex.ToString());
							}

							try
							{
								if (this.ToastClosedEvent != null)
								{
									this.ToastClosedEvent(this, new ToastClosedEventArgs()
									{
										ToastPopUp = this,
									});
								}
							}
							catch (Exception ex)
							{
								StaticLogger.LogInfo("[###] RefreshToastUI => this.Close() " + ex.ToString());
							}

							break;
						}
					case "UpdateTime":
						{
							break;
						}
				}
			}));
		}

		protected virtual void OnPropertyChanged(string propertyName)
		{
			if (this.PropertyChanged != null)
			{
				this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
			}
		}

		#endregion Methods
	}

	public class PressedEventArgs : EventArgs
	{
	}

	public class ToastActionTriggeredEventArgs : EventArgs
	{
		private Guid _phoneCallToastID = Guid.Empty;

		public Guid PhoneCallToastID
		{
			set
			{
				_phoneCallToastID = value;
			}
			get
			{
				return _phoneCallToastID;
			}
		}

		private PhoneCallToastActionType _phoneCallToastAction = PhoneCallToastActionType.None;

		public PhoneCallToastActionType PhoneCallToastAction
		{
			set
			{
				_phoneCallToastAction = value;
			}
			get
			{
				return _phoneCallToastAction;
			}
		}
	}

	public class ToastClosedEventArgs : EventArgs
	{
		private ToastPopUp _toastPopUp = null;

		public ToastPopUp ToastPopUp
		{
			set
			{
				_toastPopUp = value;
			}
			get
			{
				return _toastPopUp;
			}
		}
	}
}