﻿using Newtonsoft.Json;
using System;

namespace AirFusion.Common.JsonConverters
{
	internal class EnumConverter : JsonConverter
	{
		public override bool CanConvert(Type objectType)
		{
			return objectType.IsEnum || (objectType.IsGenericType && objectType.GetGenericTypeDefinition() == typeof(Nullable<>) && Nullable.GetUnderlyingType(objectType).IsEnum);
		}

		public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer)
		{
			if (CanConvert(objectType) == false)
			{
				throw new ArgumentException("Not supported object type: " + objectType.FullName);
			}

			bool nullable = objectType.IsGenericType && objectType.GetGenericTypeDefinition() == typeof(Nullable<>);
			Type realType = nullable ? Nullable.GetUnderlyingType(objectType) : objectType;

			if (reader.Value == null)
			{
				if (nullable)
				{
					return null;
				}
				else
				{
					throw new ArgumentException("The value is null.");
				}
			}

			switch (reader.TokenType)
			{
				case JsonToken.Null:
					if (nullable)
					{
						return null;
					}
					else
					{
						throw new ArgumentException("The value is null.");
					}
				case JsonToken.String:
					if (!string.IsNullOrEmpty(reader.Value as string))
					{
						return reader.Value.GetEnumValue(realType);
					}
					else
					{
						throw new ArgumentException("The value is empty or null.");
					}
				case JsonToken.Integer:
					return reader.Value.GetEnumValue(realType);

				default:
					throw new ArgumentException("Invalid token type.");
			}
		}

		public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
		{
			if (value != null && value is Enum)
			{
				Enum enumValue = value as Enum;
				object writeValue;
				if (enumValue.TryGetParameterValue(out writeValue))
				{
					writer.WriteValue(writeValue);
				}
				else
				{
					writer.WriteValue(enumValue.GetStringValue());
				}
			}
			else
			{
				writer.WriteNull();
			}
		}
	}
}
