﻿using Newtonsoft.Json;
using System;

namespace AirFusion.Common.JsonConverters
{
	internal class GuidConverter : JsonConverter
	{
		public override bool CanConvert(Type objectType)
		{
			return typeof(Guid) == objectType || typeof(Guid?) == objectType;
		}

		public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer)
		{
			if (CanConvert(objectType) == false)
			{
				throw new ArgumentException("Not supported object type: " + objectType.FullName);
			}

			Guid? ret = null;
			bool nullable = objectType.IsGenericType && objectType.GetGenericTypeDefinition() == typeof(Nullable<>);
			string stringValue = reader.Value != null ? reader.Value.ToString() : null;

			switch (reader.TokenType)
			{
				case JsonToken.Null:
					if (nullable)
					{
						return null;
					}
					else
					{
						throw new ArgumentException("The value is null.");
					}
				case JsonToken.String:
					if (!string.IsNullOrEmpty(stringValue))
					{
						Guid value;
						if (Guid.TryParse(stringValue, out value))
						{
							ret = value;
						}
					}
					break;

				default:
					throw new ArgumentException("Invalid token type.");
			}

			if (ret == null)
			{
				throw new ArgumentException("Invalid GUID format: " + stringValue);
			}

			return nullable && ret.HasValue ? ret : ret.Value;
		}

		public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
		{
			if (Guid.Empty.Equals(value))
			{
				writer.WriteValue("");
			}
			else
			{
				writer.WriteValue(((Guid)value).GetStringFormat());
			}
		}
	}
}
