﻿using AirFusion.Common.ProximityDetection.SoundTrigger;
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AirFusion.Common.ProximityDetection
{
	/// <summary>
	/// Represents the factory of proximity detection.
	/// </summary>
	public static class ProximityDetectionFactory
	{
		private static ConcurrentDictionary<ProximityDetectionMethod, IProximityDetection> _instanceStore;

		static ProximityDetectionFactory()
		{
			_instanceStore = new ConcurrentDictionary<ProximityDetectionMethod, IProximityDetection>();
		}

		/// <summary>
		/// Gets the instance of the specified proximity detection method.
		/// </summary>
		/// <param name="method">The method.</param>
		/// <returns></returns>
		/// <exception cref="System.NotSupportedException" />
		public static IProximityDetection GetInstance(ProximityDetectionMethod method)
		{
			switch (method)
			{
				case ProximityDetectionMethod.SoundTrigger:
					return _instanceStore.GetOrAdd(method, new SoundTriggerProximityDetectionAdapter());
			}

			throw new NotSupportedException("The specified method '" + method.ToString() + "' is not supported.");
		}
	}
}