﻿using AirFusion.Common;
using System;
using System.Diagnostics;

namespace AirFusion.Manager.DeviceManager
{
	public sealed partial class UserDeviceManager
	{
		internal sealed partial class DandelionAdapter
		{
			private void Advertise()
			{
				try
				{
					StaticLogger.TrackEnter();

					// start a stopwatch
					Stopwatch watch = new Stopwatch();
					watch.Start();

					// start pairing without getting the list
					try
					{
						Debug.Assert(this._aodController != null);
						StaticLogger.LogTrace("Call AODController.StartPairing()...");
						this._aodController.StartPairing(PairingHoldTime, (s, e) =>
						{
							// first: Get device ID list
							// second ~ nth: Get instance ID & profile (but we don't care here since this function is just advertise)

							if (e.IsCompleted)
							{
								StaticLogger.LogInfo("AODController.StartPairing() is completed in " + watch.ElapsedMilliseconds + "ms.");
								watch.Stop();
								if (!e.IsSuccess)
								{
									this.IsAdvertising = false;
									if (e.Exception != null)
									{
										StaticLogger.LogError("AODController.StartPairing() callback an error, exception:\r\n" + CommonUtils.GetExceptionMessages(e.Exception));
									}
									else
									{
										StaticLogger.LogError("AODController.StartPairing() callback an error.");
									}
								}
								else
								{
									if (this.IsAdvertising)
									{
										Advertise();
									}
								}
							}
						});
						StaticLogger.LogTrace("AODController.StartPairing() is returned without exception.");
					}
					catch (Exception ex)
					{
						StaticLogger.LogError("AODController.StartPairing() throw an exception:\r\n" + CommonUtils.GetExceptionMessages(ex));
					}
				}
				finally
				{
					StaticLogger.TrackLeave();
				}
			}
		}
	}
}