﻿using AirFusion.Common;
using Bonjour;
using System.Text;

namespace AirFusion.Manager.DeviceManager
{
	public partial class DnsSdHandler
	{
		internal enum ProcessorState
		{
			ScheduleToRun,
			Running,
			Unknown
		}

		/// <summary>
		/// Represents the data of a processor.
		/// </summary>
		public abstract class ProcessorData
		{
			/// <summary>
			/// Gets the processor ID.
			/// </summary>
			public uint ProcessorID { get; internal set; }

			/// <summary>
			/// Gets the main service.
			/// </summary>
			internal DNSSDService Service { get; set; }

			/// <summary>
			/// Gets or sets the processor.
			/// </summary>
			internal DNSSDService Processor { get; set; }

			/// <summary>
			/// Gets or sets the state.
			/// </summary>
			internal ProcessorState State { get; set; }

			/// <summary>
			/// Initializes a new instance of the <see cref="ProcessorData"/> class.
			/// </summary>
			public ProcessorData()
			{
				this.ProcessorID = uint.MaxValue;
				this.Service = null;
				this.Processor = null;
				this.State = ProcessorState.Unknown;
			}

			/// <summary>
			/// Returns a <see cref="System.String" /> that represents this instance.
			/// </summary>
			/// <returns>
			/// A <see cref="System.String" /> that represents this instance.
			/// </returns>
			public override string ToString()
			{
				StringBuilder builder = new StringBuilder();
				builder.AppendLine(this.GetType().GetFullName() + ".ProcessorID: " + this.ProcessorID);

				return builder.ToString().TrimEnd();
			}
		}
	}
}