﻿using AirFusion.Common;
using System.Text;

namespace AirFusion.Manager.DeviceManager
{
	public partial class DnsSdHandler
	{
		/// <summary>
		/// Represents the data of a record querier.
		/// </summary>
		public class RecordQuerierData : ProcessorData
		{
			/// <summary>
			/// Gets the instance name.
			/// </summary>
			public string InstanceName { get; internal set; }

			/// <summary>
			/// Gets the index of the interface.
			/// </summary>
			public uint InterfaceIndex { get; internal set; }

			/// <summary>
			/// Gets the RR type of the querier.
			/// </summary>
			public DnsSdRRType RRType { get; internal set; }

			/// <summary>
			/// Gets the query method.
			/// </summary>
			public QueryRecordMethod QueryMethod { get; internal set; }

			/// <summary>
			/// Gets the callback function.
			/// </summary>
			internal RecordQuerierCallback Callback { get; set; }

			/// <summary>
			/// Initializes a new instance of the <see cref="RecordQuerierData"/> class.
			/// </summary>
			internal RecordQuerierData()
			{
				this.InstanceName = null;
				this.InterfaceIndex = 0;
				this.RRType = DnsSdRRType.Unknown;
				this.QueryMethod = QueryRecordMethod.Default;
				this.Callback = null;
			}

			/// <summary>
			/// Initializes a new instance of the <see cref="RecordQuerierData"/> class.
			/// </summary>
			/// <param name="instanceName">The instance name.</param>
			/// <param name="interfaceIndex">The index of the interface.</param>
			/// <param name="rrType">The type of resource record.</param>
			/// <param name="queryMethod">The query method.</param>
			/// <param name="callback">The callback function.</param>
			internal RecordQuerierData(string instanceName, uint interfaceIndex, DnsSdRRType rrType, QueryRecordMethod queryMethod, RecordQuerierCallback callback)
			{
				this.InstanceName = instanceName;
				this.InterfaceIndex = interfaceIndex;
				this.RRType = rrType;
				this.QueryMethod = queryMethod;
				this.Callback = callback;
			}

			/// <summary>
			/// Returns a <see cref="System.String" /> that represents this instance.
			/// </summary>
			/// <returns>
			/// A <see cref="System.String" /> that represents this instance.
			/// </returns>
			public override string ToString()
			{
				StringBuilder builder = new StringBuilder();
				builder.AppendLine(this.GetType().GetFullName() + " (Base):\r\n" + base.ToString());
				builder.AppendLine(this.GetType().GetFullName() + ".InstanceName: " + this.InstanceName);
				builder.AppendLine(this.GetType().GetFullName() + ".InterfaceIndex: " + this.InterfaceIndex);
				builder.AppendLine(this.GetType().GetFullName() + ".RRType: " + this.RRType);
				builder.AppendLine(this.GetType().GetFullName() + ".QueryMethod: " + this.QueryMethod);

				return builder.ToString().TrimEnd();
			}
		}
	}
}