﻿using System;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;

namespace AirFusion.WebService.Adapters
{
	/// <summary>
	/// Represents the web service of ping (POST /ping).
	/// </summary>
	internal sealed class PingAdapter : WebServiceAdapter<DBNull, DBNull>
	{
		#region Definitions

		internal const string WebServiceUri = "/ping";
		internal static HttpMethod WebServiceMethod = HttpMethod.Get;

		#endregion Definitions

		#region Methods

		/// <summary>
		/// Initializes a new instance of the <see cref="PingAdapter"/> class.
		/// </summary>
		/// <param name="webServiceManager">The web service manager.</param>
		public PingAdapter(WebServicesManager webServiceManager)
			: base(webServiceManager, WebServiceUri, WebServiceMethod)
		{
		}

		/// <summary>
		/// Pings the target device asynchronously.
		/// </summary>
		/// <param name="address">The IP address of the target device.</param>
		/// <param name="port">The port number of the target device.</param>
		/// <returns>The response.</returns>
		/// <exception cref="AirFusion.WebService.Adapters.WebServiceException" />
		/// <exception cref="System.ArgumentNullException" />
		/// <exception cref="System.Exception" />
		/// <exception cref="System.InvalidOperationException" />
		/// <exception cref="System.TimeoutException" />
		public async Task<WebServiceResponse> PingAsync(IPAddress address, ushort port)
		{
			// check parameters
			if (address == null)
			{
				throw new ArgumentNullException("address");
			}

			var request = WebServiceRequest.CreateLocalRequest(
				WebServiceMethod,
				new UriBuilder(this.Scheme, address.ToString(), (int)port, WebServiceUri).Uri);

			return await this.SendRequestAsync(request, this.Timeout).ConfigureAwait(continueOnCapturedContext: false);
		}

		#endregion Methods
	}
}
