﻿using AirFusion.Common;
using System.Diagnostics;
using System.Net;
using System.Threading.Tasks;

namespace AirFusion.WebService.Adapters
{
	/// <summary>
	/// Represents the web service manager of input control (/input/*).
	/// </summary>
	internal sealed class InputControlService
	{
		#region Private/Protected/Internal Variables

		private InputDaemonStatusAdapter _daemonStatusAdapter;

		#endregion Private/Protected/Internal Variables

		#region Methods

		/// <summary>
		/// Initializes a new instance of the <see cref="InputControlService"/> class.
		/// </summary>
		/// <param name="webServiceManager">The web service manager.</param>
		public InputControlService(WebServicesManager webServiceManager)
		{
			this._daemonStatusAdapter = new InputDaemonStatusAdapter(webServiceManager);
		}

		/// <summary>
		/// Gets the status of input daemon from the target device asynchronously.
		/// </summary>
		/// <param name="address">The IP address of the target device.</param>
		/// <param name="port">The port number of the target device.</param>
		/// <returns>The response.</returns>
		/// <exception cref="AirFusion.WebService.Adapters.WebServiceException" />
		/// <exception cref="System.ArgumentNullException" />
		/// <exception cref="System.Exception" />
		/// <exception cref="System.InvalidOperationException" />
		/// <exception cref="System.TimeoutException" />
		public async Task<InputDaemonStatusAdapter.WebServiceResponse> GetStatusAsync(IPAddress address, ushort port)
		{
			try
			{
				StaticLogger.TrackEnter();

				Debug.Assert(this._daemonStatusAdapter != null);
				return await this._daemonStatusAdapter.GetStatusAsync(address, port).ConfigureAwait(continueOnCapturedContext: false);
			}
			finally
			{
				StaticLogger.TrackLeave();
			}
		}

		#endregion Methods
	}
}
