﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

namespace VNC
{
	public partial class VNCClient
	{
		// ---------------------------------------------------------------------------------

		#region Monitor File Transfer Progress Event Thread

		// For calculate total progress value
		private FTCalcProgressInfo _CalcProgressInfo;

		private static bool s_bFlag_UpdateFTProg = false;	// 檢查處理 File Transfer Progress Update 的執行緒(Thread)是否存在 => True: Existing; False: non-existence
		private static object s_oLock_UpdateFTProg = null;	// 修改 s_bFlag_UpdateFTProg 布林值時所 lock 的物件

		private static Queue<FTUpdateItem> FTProgUpdateEventQueue = null;	// 此佇列是用來暫時存放 VNCHandler 要求更新 File Transfer Progress 的 Item (會有同一個 Monitor Thread 依序取出處理並傳送給UI)
		private static object s_oLock_AccessFTPQueue = null;					// 存取 TProgUpdateEventQueue 佇列時會 lock 的物件

		/// <summary>
		/// This function will check if there is one thread is handling progress update event
		/// - If yes => Return (Do nothing)
		/// - If no  => Create a new thread to handle progress update event (Get the request events from 'FTProgUpdateEventQueue')
		/// </summary>
		private static void CreateMonitorProgressThread()
		{
			lock (s_oLock_UpdateFTProg)
			{
				if (!s_bFlag_UpdateFTProg)
				{
					s_bFlag_UpdateFTProg = true;
					Thread t = new Thread(Thread_MonitorFTProgress);
					t.Start();
				}
			}
		}

		/// <summary>
		/// This function is used for creating a thread to send the progress update event that uploads/downloads one file to Air Fusion UI
		/// </summary>
		private static void Thread_MonitorFTProgress()
		{
			WriteDebugLog(6, "++++++ Thread_MonitorFTProgress started!!!\n");
			bool bQueueEmpty = false;
			int iWaitCount = 0;

			while (true)
			{
				if (bQueueEmpty)
				{
					if (iWaitCount > 500)  // wait for 10x500 millisecond and no data.  So exit thread
						break;
					Thread.Sleep(10);
				}

				try
				{
					FTUpdateItem tmpFTUpdate = null;
					lock (s_oLock_AccessFTPQueue)
					{
						if (FTProgUpdateEventQueue.Count() <= 0)
						{
							bQueueEmpty = true;
							iWaitCount++;
							continue;
						}
						else
						{
							bQueueEmpty = false;
							iWaitCount = 0;
							tmpFTUpdate = FTProgUpdateEventQueue.Dequeue();
						}
					}

					if (FileTransferUpdateEvent == null)
					{
						continue;
					}

					// Get the FTCalcProgressInfo of VNCClient
					VNCClient tmpVncClient = s_CurVNCClientsTable[tmpFTUpdate.VncID];

					// According to the FileTransferType, to do different process
					switch (tmpFTUpdate.Type)
					{
						case FileTransferType.UploadFilesFinish:
							{
								string sFilePath = tmpVncClient._FilesPathMapTable[tmpFTUpdate.FileID];
								tmpVncClient._CalcProgressInfo.CurrentTotalProgress = 100;
								FileTransferUpdateEvent.Invoke(new FTUpdateEventArgs(VNC_FTType.Upload, tmpFTUpdate.VncID, sFilePath, 100, FileProgressState.FPS_Complete));
							}
							break;

						case FileTransferType.UploadToServer:
							{
								FTCalcProgressInfo mFTProgInfo = tmpVncClient._CalcProgressInfo;

								float fTotalProgress = mFTProgInfo.OneFilePercentUnit * (float)tmpFTUpdate.CurrentCompleteFileCount;
								fTotalProgress += (mFTProgInfo.OneFilePercentUnit * ((float)tmpFTUpdate.ProgressValue / 100.0f));

								int iNewTotalProgress = (int)fTotalProgress;
								if (iNewTotalProgress > mFTProgInfo.CurrentTotalProgress)
								{
									mFTProgInfo.CurrentTotalProgress = iNewTotalProgress;

									string sFilePath = tmpVncClient._FilesPathMapTable[tmpFTUpdate.FileID];
									FTUpdateEventArgs e = new FTUpdateEventArgs(VNC_FTType.Upload, tmpFTUpdate.VncID, sFilePath, mFTProgInfo.CurrentTotalProgress, tmpFTUpdate.State);
									FileTransferUpdateEvent.Invoke(e);
								}
							}
							break;

						case FileTransferType.DownloadFileFinish:
							FileTransferUpdateEvent.Invoke(new FTUpdateEventArgs(VNC_FTType.Download, tmpFTUpdate.VncID, tmpFTUpdate.DownloadFileName, 100, FileProgressState.FPS_Complete));
							break;

						case FileTransferType.DownloadFromServer:
							FileTransferUpdateEvent.Invoke(new FTUpdateEventArgs(VNC_FTType.Download, tmpFTUpdate.VncID, tmpFTUpdate.DownloadFileName, tmpFTUpdate.ProgressValue, tmpFTUpdate.State));
							break;
					}
				}
				catch (Exception exp)
				{
					continue;
				}
			}

			lock (s_oLock_UpdateFTProg)
			{
				s_bFlag_UpdateFTProg = false;
			}
			WriteDebugLog(6, "------ Thread_MonitorFTProgress exits!!!\n");
		}

		#endregion Monitor File Transfer Progress Event Thread

		// ---------------------------------------------------------------------------------

		#region Monitor VNC Handler Message Event Thread

		private static bool s_bFlag_SendVncMsg = false;		// Check if it is sending VNC message to Air Fusion UI (i.e. The monitor thread is existing or not)
		private static object s_oLock_SendVncMsg = null;

		private static Queue<VNCMsgItem> VNCMsgEventQueue = null;	// This queue is used for storing the request item that sends VNC Handler message
		private static object s_oLock_AccessMsgQueue = null;

		/// <summary>
		/// This function will check if there is one thread is handling VNC message event
		/// - If yes => Return (Do nothing)
		/// - If no  => Create a new thread to handle VNC message event
		/// </summary>
		private static void CreateMonitorMessageThread()
		{
			lock (s_oLock_SendVncMsg)
			{
				if (!s_bFlag_SendVncMsg)
				{
					s_bFlag_SendVncMsg = true;
					Thread t = new Thread(Thread_HandleVNCMsgEvent);
					t.Start();
				}
			}
		}

		/// <summary>
		/// This function is used for creating a thread to send VNC Handler message to Air Fusion UI
		/// </summary>
		private static void Thread_HandleVNCMsgEvent()
		{
			while (true)
			{
				try
				{
					VNCMsgItem tmpVNCMsgItem = null;
					lock (s_oLock_AccessMsgQueue)
					{
						if (VNCMsgEventQueue.Count() <= 0)
							break;
						else
							tmpVNCMsgItem = VNCMsgEventQueue.Dequeue();
					}

					// Send VNC Message
					if (VNCMessageEvent != null)
					{
						VNCMessageEventArgs e = new VNCMessageEventArgs(tmpVNCMsgItem.VncID, tmpVNCMsgItem.MsgType, tmpVNCMsgItem.FunctionCode, tmpVNCMsgItem.ErrorCode);
						if (tmpVNCMsgItem.MsgType == VNC_MSG_TYPE.MT_CommonMessage)
						{
							e.Message = tmpVNCMsgItem.MessageText;
						}

						VNCMessageEvent.Invoke(e);
					}
				}
				catch (Exception exp)
				{
					continue;
				}
			}

			lock (s_oLock_SendVncMsg)
			{
				s_bFlag_SendVncMsg = false;
			}
		}

		#endregion Monitor VNC Handler Message Event Thread
	}
}