﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace VNC
{
    public delegate void VNCMessageEventHandler(VNCMessageEventArgs e);

	public enum VNC_MSG_TYPE
	{
		MT_CommonMessage,
		MT_ErrorMessage
	}

	public enum VNC_FUN_CODE
	{
		FC_InitialVNC = 0,
		FC_StartRunVNC = 1,
		FC_HandleWinMsg = 2,
		FC_SendMouseInput = 3,
		FC_ClipboardText = 4,
		FC_FileUpload = 5,
		FC_FileUPloadCancel = 6,
		FC_FileDownload = 7,
		FC_FileTransferProc = 8,
		FC_SendIMEString = 9,
		FC_SendKeyboard = 10,

		FC_ServerRfbMsgHandler = 11
	}

	/* This error code is difined by VNC Handler (Native code c++)
	enum VNCErrorCode
	{
		....
	};
	*/
	public enum VNC_ERR_CODE
	{
		EC_NoError = 0,
		EC_FailedConnect = 1,				// Connecting to VNC server was failed for unknow reason
		EC_FailedConnect_SocketClose = 2,	// Windows Socket Error = 10038 (WSAENOTSOCK => Socket operation on nonsocket.)
		EC_FailedConnect_InvalidIP = 3,		// Windows Socket Error = 10049 (WSAEADDRNOTAVAIL => Cannot assign requested address.)
		EC_FailedConnect_TimeOut,		// Windows Socket Error = 10060 (WSAETIMEDOUT => Connection timed out.)
		EC_FailedConnect_NoServer,		// Windows Socket Errpr = 10061 (WSAECONNREFUSED => Connection refused.)
		EC_FailedConnect_NoNetwork,		// Windows Socket Error = 10065 (WSAEHOSTUNREACH => No route to host.)
		EC_ConnectionClosed,
		EC_InvalidRFBProtocol,
		EC_RFBMessageError,
		EC_InvalidVncSecurity,
		EC_SocketError,
		EC_CreateFileFailed,
		EC_VNCStopRunning,
		EC_InvalidFileSize,

		EC_OtherError
	}

	public class VNCMessageEventArgs : EventArgs
	{
		private int _VncID;
		private VNC_MSG_TYPE _MsgType;
		private VNC_FUN_CODE _VncFun;
		private VNC_ERR_CODE _ErrCode;
		private string _MsgText;

		public VNCMessageEventArgs(int iVncId, VNC_MSG_TYPE type, VNC_FUN_CODE fun, VNC_ERR_CODE errCode)
		{
			_VncID = iVncId;
			_MsgType = type;
			_VncFun = fun;
			_ErrCode = errCode;
			_MsgText = string.Empty;
		}

		public int VncID { get { return _VncID; } }
		public VNC_MSG_TYPE MsgType { get { return _MsgType; } }
		public VNC_FUN_CODE Function { get { return _VncFun; } }
		public VNC_ERR_CODE ErrorCode { get { return _ErrCode; } }
		public string Message
		{
			set { _MsgText = value; }
			get { return _MsgText; }
		}
	}

	public delegate void FileTransferUpdateEventHandler(FTUpdateEventArgs e);

	public enum VNC_FTType
	{
		Upload,
		Download
	}

	/// <summary>
	/// The type of touch event
	/// </summary>
	public enum TouchEventType
	{
		Download = 0,
		Move = 1,
		Up = 2,
		SwipeLeftToRight = 3,
		SwipeRightToLeft = 4,
		DropDownNotification = 5
	}

    public class FTUpdateEventArgs : EventArgs
    {
		private VNC_FTType _TransferType;
        private int _VncID;
        private string _FilePath;
        private int _ProgressValue;
		private FileProgressState _State;

		public FTUpdateEventArgs(VNC_FTType type, int iVncId, string sFilePath, int iProgress, FileProgressState state)
        {
			_TransferType = type;
            _VncID = iVncId;
            _FilePath = sFilePath;
            _ProgressValue = iProgress;
			_State = state;
        }

		public VNC_FTType TransferType { get { return _TransferType; } }
        public int VncID { get { return _VncID; } }
        public string FilePath { get { return _FilePath; } }
        public int Progress { get { return _ProgressValue; } }
		public FileProgressState State { get { return _State; } }
    }
}
