#include "stdhdrs.h"

#ifdef UNDER_CE
#include "omnithreadce.h"
#define SD_BOTH 0x02
#else
#include "omnithread.h"
#endif

#include "ClientConnection.h"
////#include "SessionDialog.h"
////#include "LoginAuthDialog.h"
////#include "AboutBox.h"
////#include "FileTransfer.h"
#include "commctrl.h"
#include "Exception.h"

extern "C" {
#include "vncauth.h"
#include "d3des.h"
}

#define INITIALNETBUFSIZE 4096
#define MAX_ENCODINGS 20
#define VWR_WND_CLASS_NAME _T("VNCviewer")

/*
* Macro to compare pixel formats.
*/

#define PF_EQ(x,y)							\
	((x.bitsPerPixel == y.bitsPerPixel) &&				\
	(x.depth == y.depth) &&					\
	((x.bigEndian == y.bigEndian) || (x.bitsPerPixel == 8)) &&	\
	(x.trueColour == y.trueColour) &&				\
	(!x.trueColour || ((x.redMax == y.redMax) &&			\
	(x.greenMax == y.greenMax) &&		\
	(x.blueMax == y.blueMax) &&			\
	(x.redShift == y.redShift) &&		\
	(x.greenShift == y.greenShift) &&		\
	(x.blueShift == y.blueShift))))

const rfbPixelFormat vnc8bitFormat = {8, 8, 0, 1, 7,7,3, 0,3,6,0,0};
const rfbPixelFormat vnc16bitFormat = {16, 16, 0, 1, 63, 31, 31, 0,6,11,0,0};

Log vnclog;

bool ClientConnection::sbFlag_AddClipboardChain = false;
HWND ClientConnection::sHwnd_NextViewer = NULL;

ClientConnection::ClientConnection(int index)
{
	m_vnc_index = index;
	MessageCallback = nullptr;
	Init();
	//InitWinSocket();
}

////ClientConnection::ClientConnection(LPTSTR host, int port)
////{
////	Init();
////	InitWinSocket();
////	_tcsncpy(m_host, host, MAX_HOST_NAME_LEN);
////	m_port = port;
////}

ClientConnection::~ClientConnection()
{
	return;
}

void ClientConnection::Init()
{
	m_hwnd = NULL;
	m_desktopName = NULL;
	m_port = -1;
	m_serverInitiated = false;
	m_netbuf = NULL;
	m_netbufsize = 0;
	////m_hwndNextViewer = NULL;
	m_dormant = false;
	m_passwdSet = false;

	memset(m_host, 0, sizeof(m_host));
	memset(m_password, 0, sizeof(m_password));
	memset(m_encPasswd, 0, sizeof(m_encPasswd));

	///*m_connDlg = NULL;*/

	m_enableFileTransfers = false;
	m_fileTransferDialogShown = false;
	m_pFileTransfer = new FileTransfer(this);

	////// We take the initial conn options from the application defaults
	////m_opts = m_pApp->m_options;

	m_sock = INVALID_SOCKET;
	m_bKillThread = false;
	m_threadStarted = true;
	m_running = false;
	m_pendingFormatChange = false;

	m_hScrollPos = 0; m_vScrollPos = 0;

	// Create a buffer for various network operations
	CheckBufferSize(INITIALNETBUFSIZE);

	////m_pApp->RegisterConnection(this);

	///m_initialClipboardSeen = false;
	m_iWinWidth = 0;
	m_iWinHeight = 0;
	m_iDeltaX = 0;
	m_iDeltaY = 0;
	m_fWidthRate = 1.0f;
	m_fHeightRate = 1.0f;
	m_fDPIRate = 1.0f;
	m_bIsLandscapeMode = false;
	m_bIsClintCutData = false;

	m_bSwapMouse = false;
	m_previousMouseState = 0;
	m_previousMousePos.x = -1;
	m_previousMousePos.y = -1;

	m_bFlag_OnceLMouseDown = false;
}

void ClientConnection::InitCapabilities()
{
	// Supported authentication methods
	m_authCaps.Add(rfbAuthNone, rfbStandardVendor, sig_rfbAuthNone,
		"No authentication");
	m_authCaps.Add(rfbAuthVNC, rfbStandardVendor, sig_rfbAuthVNC,
		"Standard VNC password authentication");

	// Known server->client message types
	m_serverMsgCaps.Add(rfbFileListData, rfbTightVncVendor,
		sig_rfbFileListData, "File list data");
	m_serverMsgCaps.Add(rfbFileDownloadData, rfbTightVncVendor,
		sig_rfbFileDownloadData, "File download data");
	m_serverMsgCaps.Add(rfbFileUploadCancel, rfbTightVncVendor,
		sig_rfbFileUploadCancel, "File upload cancel request");
	m_serverMsgCaps.Add(rfbFileDownloadFailed, rfbTightVncVendor,
		sig_rfbFileDownloadFailed, "File download failure notification");

	// Known client->server message types
	m_clientMsgCaps.Add(rfbFileListRequest, rfbTightVncVendor,
		sig_rfbFileListRequest, "File list request");
	m_clientMsgCaps.Add(rfbFileDownloadRequest, rfbTightVncVendor,
		sig_rfbFileDownloadRequest, "File download request");
	m_clientMsgCaps.Add(rfbFileUploadRequest, rfbTightVncVendor,
		sig_rfbFileUploadRequest, "File upload request");
	m_clientMsgCaps.Add(rfbFileUploadData, rfbTightVncVendor,
		sig_rfbFileUploadData, "File upload data");
	m_clientMsgCaps.Add(rfbFileDownloadCancel, rfbTightVncVendor,
		sig_rfbFileDownloadCancel, "File download cancel request");
	m_clientMsgCaps.Add(rfbFileUploadFailed, rfbTightVncVendor,
		sig_rfbFileUploadFailed, "File upload failure notification");

	// Supported encoding types
	m_encodingCaps.Add(rfbEncodingCopyRect, rfbStandardVendor,
		sig_rfbEncodingCopyRect, "Standard CopyRect encoding");
	m_encodingCaps.Add(rfbEncodingRRE, rfbStandardVendor,
		sig_rfbEncodingRRE, "Standard RRE encoding");
	m_encodingCaps.Add(rfbEncodingCoRRE, rfbStandardVendor,
		sig_rfbEncodingCoRRE, "Standard CoRRE encoding");
	m_encodingCaps.Add(rfbEncodingHextile, rfbStandardVendor,
		sig_rfbEncodingHextile, "Standard Hextile encoding");
	m_encodingCaps.Add(rfbEncodingZlib, rfbTridiaVncVendor,
		sig_rfbEncodingZlib, "Zlib encoding from TridiaVNC");
	m_encodingCaps.Add(rfbEncodingZlibHex, rfbTridiaVncVendor,
		sig_rfbEncodingZlibHex, "ZlibHex encoding from TridiaVNC");
	m_encodingCaps.Add(rfbEncodingTight, rfbTightVncVendor,
		sig_rfbEncodingTight, "Tight encoding by Constantin Kaplinsky");

	// Supported "fake" encoding types
	m_encodingCaps.Add(rfbEncodingCompressLevel0, rfbTightVncVendor,
		sig_rfbEncodingCompressLevel0, "Compression level");
	m_encodingCaps.Add(rfbEncodingQualityLevel0, rfbTightVncVendor,
		sig_rfbEncodingQualityLevel0, "JPEG quality level");
	m_encodingCaps.Add(rfbEncodingXCursor, rfbTightVncVendor,
		sig_rfbEncodingXCursor, "X-style cursor shape update");
	m_encodingCaps.Add(rfbEncodingRichCursor, rfbTightVncVendor,
		sig_rfbEncodingRichCursor, "Rich-color cursor shape update");
	m_encodingCaps.Add(rfbEncodingPointerPos, rfbTightVncVendor,
		sig_rfbEncodingPointerPos, "Pointer position update");
	m_encodingCaps.Add(rfbEncodingLastRect, rfbTightVncVendor,
		sig_rfbEncodingLastRect, "LastRect protocol extension");
	m_encodingCaps.Add(rfbEncodingNewFBSize, rfbTightVncVendor,
		sig_rfbEncodingNewFBSize, "Framebuffer size change");
}

void ClientConnection::InitWinSocket()
{
	// Initialise winsock
	WORD wVersionRequested = MAKEWORD(2, 0);
	WSADATA wsaData;
	if (WSAStartup(wVersionRequested, &wsaData) != 0) {
		////throw WarningException("Error creating socket");

		DWORD dwErr = WSAGetLastError();
		throw VncException("Error creating socket", EXP_SocketError, dwErr, true);
	}
}

bool ClientConnection::InitClipboard(HWND hWnd)
{
	vnclog.Print(5, "[VNC] ClientConnection::InitClipboard(): hWnd = 0x%x\n", hWnd);
	this->m_hwnd = hWnd;

	if(false == ClientConnection::sbFlag_AddClipboardChain)
	{
		ClientConnection::sHwnd_NextViewer = SetClipboardViewer(hWnd);
		DWORD dwErr = GetLastError();
		vnclog.Print(5, "[VNC] ClientConnection::InitClipboard(): Add window in Clipboard Chain => sHwnd_NextViewer = 0x%x (err = 0x%x)\n", ClientConnection::sHwnd_NextViewer, dwErr);
	}
	else
	{
		vnclog.Print(5, "[VNC] ClientConnection::InitClipboard(): bFlag_AddClipboardChain = TRUE => Do nothing! m_hwnd = 0x%x, sHwnd_NextViewer = 0x%x\n", this->m_hwnd, ClientConnection::sHwnd_NextViewer);
	}

	return true;
}

void ClientConnection::RemoveClipboard(HWND hWnd)
{
	vnclog.Print(5, "[VNC] ClientConnection::RemoveClipboard(): hWnd = 0x%x, sHwnd_NextViewer = 0x%x (sbFlag_AddClipboardChain = %d)\n", hWnd, ClientConnection::sHwnd_NextViewer, ClientConnection::sbFlag_AddClipboardChain);
	if(ClientConnection::sbFlag_AddClipboardChain)
	{
		ChangeClipboardChain(hWnd, ClientConnection::sHwnd_NextViewer);
		ClientConnection::sbFlag_AddClipboardChain = false;
	}
}

void ClientConnection::SetConnectInfo(LPTSTR host, int port, LPTSTR pw)
{
	_tcsncpy(m_host, host, MAX_HOST_NAME_LEN);
	m_port = port;
	_tcsncpy(m_password, pw, MAX_SIZE_PASSWORD);
}

//
// Run() creates the connection if necessary, does the initial negotiations
// and then starts the thread running which does the output (update) processing.
// If Run throws an Exception, the caller must delete the ClientConnection object.
//

void ClientConnection::Run()
{
	vnclog.Print(5, _T("[VNC] ClientConnection::Run(): Start\n"));
	InitWinSocket();

	// Connect if we're not already connected
	if (m_sock == INVALID_SOCKET)
	{
		vnclog.Print(5, _T("[VNC] ClientConnection::Run(): Call Connect() ...\n"));
		Connect();
	}

	SetSocketOptions();

	NegotiateProtocolVersion();

	PerformAuthentication();

	vnclog.Print(5, _T("[VNC] ClientConnection::Run(): Send client initialization\n"));
	SendClientInit();

	vnclog.Print(5, _T("[VNC] ClientConnection::Run(): Read server initialization\n"));
	ReadServerInit();

	// Only for protocol version 3.7t
	if (m_tightVncProtocol) {
		// Determine which protocol messages and encodings are supported.
		ReadInteractionCaps();
		// Enable file transfers only if the server supports that.
		m_enableFileTransfers = false;
		if ( m_clientMsgCaps.IsEnabled(rfbFileListRequest) &&
			m_serverMsgCaps.IsEnabled(rfbFileListData) ) {
				m_enableFileTransfers = true;
		}
	}
	else
	{
		vnclog.Print(0, _T("[ERROR] ClientConnection::Run(): Invalid protocol version !!"));
		throw VncException("[ClientConnection::Run] Invalid protocol version.", EXP_CustomError, EC_InvalidRFBProtocol, true);
	}

	m_rfbKeySym = std::auto_ptr<RfbKeySym>(new RfbKeySym(this, &vnclog));

	// This starts the worker thread.
	// The rest of the processing continues in run_undetached.
	vnclog.Print(5, _T("[VNC] ClientConnection::Run(): To start the worker thread => run_undetached()\n"));
	start_undetached();
	vnclog.Print(5, _T("[VNC] ClientConnection::Run(): END\n"));
}

void ClientConnection::Close()
{
	vnclog.Print(5, _T("[VNC] ClientConnection::Close(): Close VNC !!\n"));
	omni_mutex_lock l(m_callbackMutex);
	this->MessageCallback = nullptr;

	KillThread();

	// remove this window (m_hwnd) from the clipboard viewer chain
	if(this->m_hwnd != NULL){
		////ChangeClipboardChain(this->m_hwnd, this->m_hwndNextViewer); //[Workaround]
		this->m_hwnd = NULL;
		////ClientConnection::sHwnd_NextViewer = NULL;
	}

	//Rmove to fix to close the VNC will crash
	//if (m_desktopName != NULL) delete [] m_desktopName;
	//if (m_netbuf != NULL) delete [] m_netbuf;

	// Clean up winsock
	WSACleanup();
}

// Closing down the connection.
// Close the socket, kill the thread.
void ClientConnection::KillThread()
{
	m_bKillThread = true;
	vnclog.Print(5, "[Warning] ClientConnection::KillThread() -> m_running = false;\n");
	m_running = false;

	if (m_sock != INVALID_SOCKET) {
		shutdown(m_sock, SD_BOTH);
		closesocket(m_sock);
		m_sock = INVALID_SOCKET;
	}
}

void ClientConnection::Connect()
{
	struct sockaddr_in thataddr;
	int res;

	// Msg: Connection initiated
	m_sock = socket(PF_INET, SOCK_STREAM, 0);

	if (m_sock == INVALID_SOCKET){
		DWORD dwErr = WSAGetLastError();
		vnclog.Print(1, _T("[ERROR] ClientConnection::Connect(): Initial socket failed (err code = %d)\n"), dwErr);
		throw VncException("Error creating socket", EXP_SocketError, dwErr, true);
	}

	// The host may be specified as a dotted address "a.b.c.d"
	// Try that first
	thataddr.sin_addr.s_addr = inet_addr(m_host);

	// If it wasn't one of those, do gethostbyname
	if (thataddr.sin_addr.s_addr == INADDR_NONE) {
		LPHOSTENT lphost;
		lphost = gethostbyname(m_host);

		if (lphost == NULL) {
			char msg[512];
			sprintf(msg, "Failed to get server address (%s)", m_host);
			vnclog.Print(1, _T("[ERROR] ClientConnection::Connect(): %s\n"), msg);

			throw VncException(msg, EXP_CustomError, EC_FailedConnect_InvalidIP, true);
		};
		thataddr.sin_addr.s_addr = ((LPIN_ADDR) lphost->h_addr)->s_addr;
	};

	thataddr.sin_family = AF_INET;
	thataddr.sin_port = htons(m_port);
	vnclog.Print(5,  _T("[VNC] ClientConnection::Connect(): Server IP = %s, Port = %d\n"), m_host, m_port);
	res = connect(m_sock, (LPSOCKADDR) &thataddr, sizeof(thataddr));
	if (res == SOCKET_ERROR) {
		DWORD dwErr = WSAGetLastError();
		char msg[512];
		sprintf(msg, "Failed to connect to server (IP = %s, Port = %d): Error = %d", m_host, m_port, (int)dwErr);
		vnclog.Print(1, _T("[ERROR] ClientConnection::Connect(): %s\n"), msg);

		// Send VNC error code to Air Fusion UI
		if(dwErr == WSAENOTSOCK)
			throw VncException(msg, EXP_CustomError, EC_FailedConnect_SocketClose, true);
		else if(dwErr == WSAEADDRNOTAVAIL)
			throw VncException(msg, EXP_CustomError, EC_FailedConnect_InvalidIP, true);
		else if(dwErr == WSAETIMEDOUT)
			throw VncException(msg, EXP_CustomError, EC_FailedConnect_TimeOut, true);
		else if(dwErr == WSAECONNREFUSED)
			throw VncException(msg, EXP_CustomError, EC_FailedConnect_NoServer, true);
		else if(dwErr == WSAEHOSTUNREACH)
			throw VncException(msg, EXP_CustomError, EC_FailedConnect_NoNetwork, true);
		else
			throw VncException(msg, EXP_CustomError, EC_FailedConnect, true);
	}

	// Msg: Connection established
	vnclog.Print(0, _T("[VNC] ClientConnection::Connect(): Completed !!\n"));
}

void ClientConnection::SetSocketOptions() {
	// Disable Nagle's algorithm
	BOOL nodelayval = TRUE;
	if (setsockopt(m_sock, IPPROTO_TCP, TCP_NODELAY, (const char *) &nodelayval, sizeof(BOOL)))
	{
		DWORD dwErr = WSAGetLastError();
		vnclog.Print(1, "[ERROR] ClientConnection::SetSocketOptions(): Failed to set socket option !! (err code = %d)\n", dwErr);
		throw VncException("Failed to set socket option", EXP_SocketError, dwErr, true);
	}

	// Enable KeepAlive
	DWORD dwBytesRet=0;
	//The default settings when a TCP socket is initialized sets the keep-alive timeout to 2 hours
	// and the keep-alive interval to 1 second.
	tcp_keepalive alive;
	alive.onoff = TRUE;
	alive.keepalivetime = 5000;
	alive.keepaliveinterval = 2000;
	//Note: On Windows Vista and later, the number of keep-alive probes (data retransmissions)
	// is set to 10 and cannot be changed.
	if (SOCKET_ERROR == WSAIoctl(m_sock, SIO_KEEPALIVE_VALS, &alive, sizeof(alive), NULL, 0, &dwBytesRet, NULL, NULL))
	{
		DWORD dwErr = WSAGetLastError();
		vnclog.Print(1, "[ERROR] ClientConnection::SetSocketOptions(): Fao;ed tp run WSAIotcl(SIO_KEEPALIVE_VALS)!! (err code = %d)\n", dwErr);
		throw VncException("Failed to set socket option (KeepAlive)", EXP_SocketError, dwErr, true);
	}
}

void ClientConnection::NegotiateProtocolVersion()
{
	rfbProtocolVersionMsg pv;

	ReadExact(pv, sz_rfbProtocolVersionMsg);

	pv[sz_rfbProtocolVersionMsg] = 0;

	////if (m_connDlg != NULL)
	////	m_connDlg->SetStatus("Server protocol version received");

	// XXX This is a hack.  Under CE we just return to the server the
	// version number it gives us without parsing it.
	// Too much hassle replacing sscanf for now. Fix this!
#ifdef UNDER_CE
	m_minorVersion = 8;
#else
	int majorVersion, minorVersion;
	if (sscanf(pv, rfbProtocolVersionFormat, &majorVersion, &minorVersion) != 2) {
		vnclog.Print(1, _T("[ERROR] ClientConnection::NegotiateProtocolVersion(): RFB protocol doesn't support! (Server's RFB protocol = %s)\n"), pv);
		throw VncException("[ClientConnection::NegotiateProtocolVersion] Invalid protocol.", EXP_CustomError, EC_InvalidRFBProtocol, true);
	}

	//vnclog.Print(0, _T("[VNC] ClientConnection::NegotiateProtocolVersion(): RFB server supports protocol version 3.%d\n"), minorVersion);

	if (majorVersion == 3 && minorVersion >= 8) {
		m_minorVersion = 8;
	} else if (majorVersion == 3 && minorVersion == 7) {
		m_minorVersion = 7;
	} else {
		m_minorVersion = 3;
	}

	m_tightVncProtocol = false;

	sprintf(pv, rfbProtocolVersionFormat, 3, m_minorVersion);
#endif

	WriteExact(pv, sz_rfbProtocolVersionMsg);

	////if (m_connDlg != NULL)
	////	m_connDlg->SetStatus("Protocol version negotiated");

	vnclog.Print(0, _T("[VNC] ClientConnection::NegotiateProtocolVersion(): Connected to RFB server, using protocol version 3.%d\n"), m_minorVersion);
}

//
// Negotiate authentication scheme and authenticate if necessary
//

void ClientConnection::PerformAuthentication()
{
	int secType;
	if (m_minorVersion >= 7) {
		secType = SelectSecurityType();
	} else {
		secType = ReadSecurityType();
	}

	switch (secType) {
	case rfbSecTypeNone:
		{
			Authenticate(rfbAuthNone);
			m_authScheme = rfbAuthNone;
			break;
		}
	case rfbSecTypeVncAuth:
		{
			Authenticate(rfbAuthVNC);
			m_authScheme = rfbAuthVNC;
			break;
		}
	case rfbSecTypeTight:
		{
			m_tightVncProtocol = true;
			InitCapabilities();
			SetupTunneling();
			PerformAuthenticationTight();
			break;
		}
	default:	// should never happen
		{
			vnclog.Print(0, _T("[ERROR] ClientConnection::PerformAuthentication(): Invalid security type\n"));
			throw VncException("[ClientConnection::PerformAuthentication] Internal error: Invalid security type", EXP_CustomError, EC_InvalidVncSecurity, true);
		}
	}
}

//
// Read security type from the server (protocol 3.3) [CanRemoved]
//

int ClientConnection::ReadSecurityType()
{
	// Read the authentication scheme.
	CARD32 secType;
	ReadExact((char *)&secType, sizeof(secType));
	secType = Swap32IfLE(secType);

	if (secType == rfbSecTypeInvalid)
	{
		// [ToDo] Send Warning Event
		// throw WarningException(ReadFailureReason());
	}

	if (secType != rfbSecTypeNone && secType != rfbSecTypeVncAuth) {
		vnclog.Print(0, _T("[ERROR] ClientConnection::ReadSecurityType(): Unknown security type from RFB server: %d\n"), (int)secType);

		throw ErrorException("Unknown security type requested!");
	}

	////if (m_connDlg != NULL)
	////	m_connDlg->SetStatus("Security type received");

	return (int)secType;
}

//
// Select security type from the server's list (protocol 3.7 and above)
//

int ClientConnection::SelectSecurityType()
{
	// Read the list of secutiry types.
	CARD8 nSecTypes;
	ReadExact((char *)&nSecTypes, sizeof(nSecTypes));
	if (nSecTypes == 0)
	{
		vnclog.Print(0, _T("[ERROR] ClientConnection::SelectSecurityType(): Failed to read the list of secutiry types\n"));
		throw WarningException(ReadFailureReason());
	}

	char *secTypeNames[] = {"None", "VncAuth"};
	CARD8 knownSecTypes[] = {rfbSecTypeNone, rfbSecTypeVncAuth};
	int nKnownSecTypes = sizeof(knownSecTypes);
	CARD8 *secTypes = new CARD8[nSecTypes];
	ReadExact((char *)secTypes, nSecTypes);
	CARD8 secType = rfbSecTypeInvalid;

	////if (m_connDlg != NULL)
	////	m_connDlg->SetStatus("List of security types received");

	// Find out if the server supports TightVNC protocol extensions
	int j;
	for (j = 0; j < (int)nSecTypes; j++) {
		if (secTypes[j] == rfbSecTypeTight) {
			secType = rfbSecTypeTight;
			WriteExact((char *)&secType, sizeof(secType));

			////if (m_connDlg != NULL)
			////	m_connDlg->SetStatus("TightVNC protocol extensions enabled");

			vnclog.Print(8, _T("Enabling TightVNC protocol extensions\n"));
			return rfbSecTypeTight;
		}
	}

	// Find first supported security type
	for (j = 0; j < (int)nSecTypes; j++) {
		for (int i = 0; i < nKnownSecTypes; i++) {
			if (secTypes[j] == knownSecTypes[i]) {
				secType = secTypes[j];
				WriteExact((char *)&secType, sizeof(secType));

				////if (m_connDlg != NULL)
				////	m_connDlg->SetStatus("Security type requested");

				vnclog.Print(8, _T("Choosing security type %s(%d)\n"),
					secTypeNames[i], (int)secType);
				break;
			}
		}
		if (secType != rfbSecTypeInvalid) break;
	}

	if (secType == rfbSecTypeInvalid) {
		vnclog.Print(0, _T("[ERROR] ClientConnection::SelectSecurityType(): Server did not offer supported security type\n"));

		throw ErrorException("Server did not offer supported security type!");
	}

	return (int)secType;
}

//
// Setup tunneling (protocol 3.7t, 3.8t)
//

void ClientConnection::SetupTunneling()
{
	rfbTunnelingCapsMsg caps;
	ReadExact((char *)&caps, sz_rfbTunnelingCapsMsg);
	caps.nTunnelTypes = Swap32IfLE(caps.nTunnelTypes);

	if (caps.nTunnelTypes) {
		ReadCapabilityList(&m_tunnelCaps, caps.nTunnelTypes);

		// [ToDo] Program Message
		////if (m_connDlg != NULL)
		////	m_connDlg->SetStatus("List of tunneling capabilities received");

		// We cannot do tunneling yet.
		CARD32 tunnelType = Swap32IfLE(rfbNoTunneling);
		WriteExact((char *)&tunnelType, sizeof(tunnelType));

		// [ToDo] Program Message
		////if (m_connDlg != NULL)
		////	m_connDlg->SetStatus("Tunneling type requested");
	}
}

//
// Negotiate authentication scheme (protocol 3.7t, 3.8t)
//

void ClientConnection::PerformAuthenticationTight()
{
	rfbAuthenticationCapsMsg caps;
	ReadExact((char *)&caps, sz_rfbAuthenticationCapsMsg);
	caps.nAuthTypes = Swap32IfLE(caps.nAuthTypes);

	////if (m_connDlg != NULL)
	////	m_connDlg->SetStatus("Header of authentication capability list received");

	if (!caps.nAuthTypes) {
		vnclog.Print(0, _T("[VNC] ClientConnection::PerformAuthenticationTight(): No authentication needed\n"));

		////if (m_connDlg != NULL)
		////	m_connDlg->SetStatus("No authentication needed");

		Authenticate(rfbAuthNone);
		m_authScheme = rfbAuthNone;
	} else {
		ReadCapabilityList(&m_authCaps, caps.nAuthTypes);

		////if (m_connDlg != NULL)
		////	m_connDlg->SetStatus("Authentication capability list received");

		if (!m_authCaps.NumEnabled()) {
			vnclog.Print(0, _T("[ERROR] ClientConnection::PerformAuthenticationTight(): No suitable authentication schemes offered by the server\n"));
			throw ErrorException("No suitable authentication schemes offered by the server");
		}

		// Use server's preferred authentication scheme.
		CARD32 authScheme = m_authCaps.GetByOrder(0);
		authScheme = Swap32IfLE(authScheme);
		WriteExact((char *)&authScheme, sizeof(authScheme));
		authScheme = Swap32IfLE(authScheme);	// convert it back

		////if (m_connDlg != NULL)
		////	m_connDlg->SetStatus("Authentication scheme requested");

		Authenticate(authScheme);
		m_authScheme = authScheme;
	}
}

// The definition of a function implementing some authentication scheme.
// For an example, see ClientConnection::AuthenticateVNC, below.

typedef bool (ClientConnection::*AuthFunc)(char *, int);

// A wrapper function for different authentication schemes.

void ClientConnection::Authenticate(CARD32 authScheme)
{
	AuthFunc authFuncPtr;

	// Uncomment this if the "Connecting..." dialog box should be
	// closed prior to authentication.
	/***
	if (m_connDlg != NULL) {
	delete m_connDlg;
	m_connDlg = NULL;
	}
	***/

	switch(authScheme) {
	case rfbAuthNone:
		authFuncPtr = &ClientConnection::AuthenticateNone;
		break;
	case rfbAuthVNC:
		authFuncPtr = &ClientConnection::AuthenticateVNC;
		break;
	default:
		vnclog.Print(0, _T("[ERROR] ClientConnection::Authenticate(): Unknown authentication scheme: %d\n"), (int)authScheme);
		throw ErrorException("Unknown authentication scheme!");
	}

	vnclog.Print(0, _T("[VNC] ClientConnection::Authenticate(): Authentication scheme: %s\n"), m_authCaps.GetDescription(authScheme));

	const int errorMsgSize = 256;
	CheckBufferSize(errorMsgSize);
	char *errorMsg = m_netbuf;
	bool wasError = !(this->*authFuncPtr)(errorMsg, errorMsgSize);

	// Report authentication error.
	if (wasError) {
		vnclog.Print(0, _T("[ERROR] ClientConnection::Authenticate(): %s\n"), errorMsg);
		throw AuthException(errorMsg);
	}

	CARD32 authResult;
	if (authScheme == rfbAuthNone && m_minorVersion < 8) {
		// In protocol versions prior to 3.8, "no authentication" is a
		// special case - no "security result" is sent by the server.
		authResult = rfbAuthOK;
	} else {
		ReadExact((char *) &authResult, 4);
		authResult = Swap32IfLE(authResult);
	}

	switch (authResult) {
	case rfbAuthOK:
		// [ToDo] Program Message
		////if (m_connDlg != NULL)
		////	m_connDlg->SetStatus("Authentication successful");

		vnclog.Print(0, _T("[VNC] ClientConnection::Authenticate(): Authentication successful\n"));
		return;
	case rfbAuthFailed:
		if (m_minorVersion >= 8) {
			errorMsg = ReadFailureReason();
		} else {
			errorMsg = "Authentication failure";
		}
		break;
	case rfbAuthTooMany:
		errorMsg = "Authentication failure, too many tries";
		break;
	default:
		_snprintf(m_netbuf, 256, "Unknown authentication result (%d)",
			(int)authResult);
		errorMsg = m_netbuf;
		break;
	}

	// Report authentication failure.
	vnclog.Print(0, _T("[ERROR] ClientConnection::Authenticate(): %s\n"), errorMsg);

	// [ToDo] Error Message
	////if (m_connDlg != NULL)
	////	m_connDlg->SetStatus(errorMsg);

	throw AuthException(errorMsg);
}

// "Null" authentication.

bool ClientConnection::AuthenticateNone(char *errBuf, int errBufSize)
{
	return true;
}

// The standard VNC authentication.
//
// An authentication function should return false on error and true if
// the authentication process was successful. Note that returning true
// does not mean that authentication was passed by the server, the
// server's result will be received and analyzed later.
// If false is returned, then a text error message should be copied
// to errorBuf[], no more than errBufSize bytes should be copied into
// that buffer.

bool ClientConnection::AuthenticateVNC(char *errBuf, int errBufSize)
{
	CARD8 challenge[CHALLENGESIZE];
	ReadExact((char *)challenge, CHALLENGESIZE);

	char passwd[MAXPWLEN + 1];
	///strcpy(passwd, TEST_PASSWORD);
	strcpy(passwd, m_password);

	vncEncryptBytes(challenge, passwd);

	/* Lose the plain-text password from memory */
	memset(passwd, 0, strlen(passwd));

	WriteExact((char *) challenge, CHALLENGESIZE);

	return true;
}

void ClientConnection::SendClientInit()
{
	rfbClientInitMsg ci;
	ci.shared = m_opts.m_Shared;

	WriteExact((char *)&ci, sz_rfbClientInitMsg);
}

void ClientConnection::ReadServerInit()
{
	ReadExact((char *)&m_si, sz_rfbServerInitMsg);

	m_si.framebufferWidth = Swap16IfLE(m_si.framebufferWidth);
	m_si.framebufferHeight = Swap16IfLE(m_si.framebufferHeight);
	m_si.format.redMax = Swap16IfLE(m_si.format.redMax);
	m_si.format.greenMax = Swap16IfLE(m_si.format.greenMax);
	m_si.format.blueMax = Swap16IfLE(m_si.format.blueMax);
	m_si.nameLength = Swap32IfLE(m_si.nameLength);

	m_iWinWidth = (int)m_si.framebufferWidth;
	m_iWinHeight = (int)m_si.framebufferHeight;

	m_desktopName = new TCHAR[m_si.nameLength + 2];

#ifdef UNDER_CE
	char *deskNameBuf = new char[m_si.nameLength + 2];

	ReadString(deskNameBuf, m_si.nameLength);

	MultiByteToWideChar( CP_ACP, MB_PRECOMPOSED, deskNameBuf, m_si.nameLength, m_desktopName, m_si.nameLength+1);
	delete deskNameBuf;
#else
	ReadString(m_desktopName, m_si.nameLength);
#endif

	vnclog.Print(0, _T("[VNC] ClientConnection::ReadServerInit(): Desktop name \"%s\"\n"),m_desktopName);
	vnclog.Print(1, _T("[VNC] ClientConnection::ReadServerInit(): Geometry %d x %d depth %d\n"), m_si.framebufferWidth, m_si.framebufferHeight, m_si.format.depth );

	////SetWindowText(m_hwnd1, m_desktopName);
	////SetWindowText(m_hwnd1, m_desktopName);
	////SizeWindow(true);
}

//
// In protocols 3.7t/3.8t, the server informs us about supported
// protocol messages and encodings. Here we read this information.
//

void ClientConnection::ReadInteractionCaps()
{
	// Read the counts of list items following
	rfbInteractionCapsMsg intr_caps;
	ReadExact((char *)&intr_caps, sz_rfbInteractionCapsMsg);
	intr_caps.nServerMessageTypes = Swap16IfLE(intr_caps.nServerMessageTypes);
	intr_caps.nClientMessageTypes = Swap16IfLE(intr_caps.nClientMessageTypes);
	intr_caps.nEncodingTypes = Swap16IfLE(intr_caps.nEncodingTypes);

	// Read the lists of server- and client-initiated messages
	ReadCapabilityList(&m_serverMsgCaps, intr_caps.nServerMessageTypes);
	ReadCapabilityList(&m_clientMsgCaps, intr_caps.nClientMessageTypes);
	ReadCapabilityList(&m_encodingCaps, intr_caps.nEncodingTypes);

	// Msg: Interaction capability list received
}

//
// Read the list of rfbCapabilityInfo structures and enable corresponding
// capabilities in the specified container. The count argument specifies how
// many records to read from the socket.
//

void ClientConnection::ReadCapabilityList(CapsContainer *caps, int count)
{
	rfbCapabilityInfo msginfo;
	for (int i = 0; i < count; i++) {
		ReadExact((char *)&msginfo, sz_rfbCapabilityInfo);
		msginfo.code = Swap32IfLE(msginfo.code);
		caps->Enable(&msginfo);
	}
}

#ifndef UNDER_CE

LRESULT CALLBACK ClientConnection::WndProc(HWND hwnd, UINT iMsg,
										   WPARAM wParam, LPARAM lParam)
{
	switch (iMsg) {
	case WM_PAINT:
		return 0;
	case WM_TIMER:
		return 0;
	case WM_LBUTTONDBLCLK:
	case WM_LBUTTONDOWN:
	case WM_LBUTTONUP:
	case WM_MBUTTONDOWN:
	case WM_MBUTTONUP:
	case WM_RBUTTONDOWN:
	case WM_RBUTTONUP:
	case WM_MOUSEWHEEL:
	case WM_MOUSEMOVE:
	case WM_MOUSELEAVE:
		{
			if (!this->m_running)
				return 0;

			if(hwnd != m_hwnd)	//if(GetFocus() != m_hwnd)
				return 0;

			SetFocus(hwnd);

			if(iMsg == WM_LBUTTONDOWN)
			{
				m_bFlag_OnceLMouseDown = true;
			}
			else if(iMsg == WM_LBUTTONUP)
			{
				m_bFlag_OnceLMouseDown = false;
			}

			unsigned char mouseButtons = 0;
			mouseButtons |= LOWORD(wParam) & MK_RBUTTON ? MOUSE_RDOWN : 0;
			mouseButtons |= LOWORD(wParam) & MK_MBUTTON ? MOUSE_MDOWN : 0;
			mouseButtons |= LOWORD(wParam) & MK_LBUTTON ? MOUSE_LDOWN : 0;

			if((iMsg == WM_MOUSEMOVE) && (mouseButtons == 0))
				return 0;

			// Translate position from LPARAM to POINT.
			POINTS points = MAKEPOINTS(lParam);
			POINT point;
			point.x = points.x;
			point.y = points.y;

			if(iMsg == WM_MOUSELEAVE)
			{
				if(m_bFlag_OnceLMouseDown)
				{
					point.x = m_previousMousePos.x;
					point.y = m_previousMousePos.y;

					SendPointerEvent(point.x, point.y, 0);
					m_bFlag_OnceLMouseDown = false;
					return 0;
				}
				else
					return 0;
			}

			unsigned short wheelSpeed = 0;
			if (iMsg == WM_MOUSEWHEEL)
			{
				// Get speed wheel and set mouse button.
				signed short wheelSignedSpeed = static_cast<signed short>(HIWORD(wParam));
				if (wheelSignedSpeed < 0)
				{
					mouseButtons |= MOUSE_WDOWN;
					wheelSpeed = - wheelSignedSpeed / WHEEL_DELTA;
				}
				else
				{
					mouseButtons |= MOUSE_WUP;
					wheelSpeed = wheelSignedSpeed / WHEEL_DELTA;
				}

				// In some cases wheelSignedSpeed can be smaller than the WHEEL_DELTA,
				// then wheelSpeed set to 1, but not 0.
				if (wheelSpeed == 0) {
					wheelSpeed = 1;
				}

				// If windows-message is WHEEL, then need to translate screen coordinate to client.
				if (!ScreenToClient(hwnd, &point))
				{
					point.x = -1;
					point.y = -1;
				}
			}

			if(point.x > 0)
			{
				point.x = (point.x * m_fDPIRate) - m_iDeltaX;
				if(point.x < 0) return 0;
			}

			if(point.y > 0)
			{
				point.y = (point.y * m_fDPIRate) - m_iDeltaY;
				if(point.y < 0) return 0;
			}

			this->onMouse(mouseButtons, static_cast<unsigned short>(wheelSpeed), point);

			return 0;
		}
	case WM_KEYDOWN:
	case WM_KEYUP:
	case WM_SYSKEYDOWN:
	case WM_SYSKEYUP:
		{
			if (!this->m_running)
			{
				if(iMsg == WM_KEYDOWN)
				{
					SendVncMessage(MT_ErrorMessage, FC_SendKeyboard, EC_ConnectionClosed, "[Error] VNC Client stopped running !!");
				}
				return 0;
			}

			this->onKey(wParam, lParam);
			return 0;
		}
	case WM_IME_CHAR:	// oJkͪr
		{
			// [NotUse][Because WPF couldn't receive WM_IME_CHAR message][Workaround]
			return 0;
		}
	case WM_CHAR:
		return 0;
	case WM_SYSCHAR:
#ifdef UNDER_CE
		{
			int key = wParam;
			vnclog.Print(4,_T("CHAR msg : %02x\n"), key);
			// Control keys which are in the Keymap table will already
			// have been handled.
			if (key == 0x0D ||		// return
				key == 0x20 ||		// space
				key == 0x08)		// backspace
				return 0;

			if (key < 32) key += 64;	// map ctrl-keys onto alphabet
			if (key > 32 && key < 127) {
				this->SendKeyEvent(wParam & 0xff, true);
				this->SendKeyEvent(wParam & 0xff, false);
			}
			return 0;
		}
#endif
	case WM_DEADCHAR:
	case WM_SYSDEADCHAR:
		return 0;
	case WM_SETFOCUS:
		////if (_this->InFullScreenMode())
		////	SetWindowPos(hwnd, HWND_TOPMOST, 0,0,100,100, SWP_NOMOVE | SWP_NOSIZE);
		return 0;
		// Cacnel modifiers when we lose focus
	case WM_KILLFOCUS:
		{
			if (!this->m_running) return 0;

			//vnclog.Print(6, _T("Losing focus - cancelling modifiers\n"));
			return 0;
		}
	case WM_QUERYNEWPALETTE:
		{
			return TRUE;
		}
	case WM_PALETTECHANGED:
		{
			break;
		}

#ifndef UNDER_CE

	case WM_SETCURSOR:
		{
			// if we have the focus, let the cursor change as normal
			if(hwnd == m_hwnd)		//if (GetFocus() == hwnd)
				break;

			// if not, set to default system cursor
			SetCursor( LoadCursor(NULL, IDC_ARROW));
			return 0;
		}

	case WM_DRAWCLIPBOARD:
		{
			if (this->m_running)
			{
				this->ProcessLocalClipboardChange();
			}
			return 0;
		}
	case WM_CHANGECBCHAIN:
		{
			// The clipboard chain is changing
			HWND hWndRemove = (HWND) wParam;     // handle of window being removed
			HWND hWndNext = (HWND) lParam;       // handle of next window in chain

			// If next window is closing, update our pointer.
			if (hWndRemove == ClientConnection::sHwnd_NextViewer)
				ClientConnection::sHwnd_NextViewer = hWndNext;
			// Otherwise, pass the message to the next link.
			else if ( ClientConnection::sHwnd_NextViewer != NULL)
				::SendMessage( ClientConnection::sHwnd_NextViewer, WM_CHANGECBCHAIN, (WPARAM) hWndRemove,  (LPARAM) hWndNext );

			return 0;
		}

#endif
	}
}

////// ProcessPointerEvent handles the delicate case of emulating 3 buttons
////// on a two button mouse, then passes events off to SubProcessPointerEvent.
////
////void
////ClientConnection::ProcessPointerEvent(int x, int y, DWORD keyflags, UINT msg)
////{
//...
////}

////// SubProcessPointerEvent takes windows positions and flags and converts
////// them into VNC ones.
////
////inline void
////ClientConnection::SubProcessPointerEvent(int x, int y, DWORD keyflags)
////{
//...
////}

bool ClientConnection::onMouse(unsigned char mouseButtons, unsigned short wheelSpeed, POINT position)
{
	// If swap of mouse button is enabled, then swap button.
	if (m_bSwapMouse && mouseButtons)
	{
		bool bSecond = !!(mouseButtons & MOUSE_MDOWN);
		bool bThird  = !!(mouseButtons & MOUSE_RDOWN);
		mouseButtons &= ~(MOUSE_MDOWN | MOUSE_RDOWN);
		if (bSecond) {
			mouseButtons |= MOUSE_RDOWN;
		}
		if (bThird) {
			mouseButtons |= MOUSE_MDOWN;
		}
	}

	// Check if it is mouse wheel message
	if(wheelSpeed > 0)
	{
		// Check if the mouse position is in AirFusion UI
		if(position.x < 0 || position.y < 0)
			return true;

		if(position.x > m_iWinWidth || position.y > m_iWinHeight)
			return true;
	}

	UINT8 buttons = mouseButtons;
	Point pos;

	// Translate coordinates from the Viewer Window to Smartphone Screen
	pos.x = (LONG)((float) position.x * m_fWidthRate);
	pos.y = (LONG)((float) position.y * m_fHeightRate);

	// Check the position of Y is over the max height of screen size of phone
	if((pos.y >= m_si.framebufferHeight) && (buttons & MOUSE_LDOWN == MOUSE_LDOWN))
	{
		vnclog.Print(5, "[VNC] ClientConnection::onMouse(): pos.y = %d (MOUSE_LDOWN) => Not send mouse down event to phone !!\n", pos.y);
		return true;
	}

	// If posititon of mouse isn't change, then don't send event to server.
	if (buttons != m_previousMouseState || !pos.isEqualTo(&m_previousMousePos))
	{
		int wheelMask = MOUSE_WUP | MOUSE_WDOWN;

		// Update previously position of mouse.
		m_previousMouseState = buttons & ~wheelMask;
		m_previousMousePos = pos;

		if ((buttons & wheelMask) == 0)
		{
			// Send position of cursor and state of buttons one time.
			SendPointerEvent(pos.x, pos.y, buttons);
		}
		else
		{
			// Send position of cursor and state of buttons wheelSpeed times.
			for (int i = 0; i < wheelSpeed; i++) {
				SendPointerEvent(pos.x, pos.y, buttons);
				SendPointerEvent(pos.x, pos.y, buttons & ~wheelMask);
			}
		} // wheel
	}

	return true;
}

//
// ProcessKeyEvent
//
// Normally a single Windows key event will map onto a single RFB
// key message, but this is not always the case.  Much of the stuff
// here is to handle AltGr (=Ctrl-Alt) on international keyboards.
// Example cases:
//
//    We want Ctrl-F to be sent as:
//      Ctrl-Down, F-Down, F-Up, Ctrl-Up.
//    because there is no keysym for ctrl-f, and because the ctrl
//    will already have been sent by the time we get the F.
//
//    On German keyboards, @ is produced using AltGr-Q, which is
//    Ctrl-Alt-Q.  But @ is a valid keysym in its own right, and when
//    a German user types this combination, he doesn't mean Ctrl-@.
//    So for this we will send, in total:
//
//      Ctrl-Down, Alt-Down,
//                 (when we get the AltGr pressed)
//
//      Alt-Up, Ctrl-Up, @-Down, Ctrl-Down, Alt-Down
//                 (when we discover that this is @ being pressed)
//
//      Alt-Up, Ctrl-Up, @-Up, Ctrl-Down, Alt-Down
//                 (when we discover that this is @ being released)
//
//      Alt-Up, Ctrl-Up
//                 (when the AltGr is released)

////inline void ClientConnection::ProcessKeyEvent(int virtkey, DWORD keyData)
////{
////	....
////}

//
// Follow the code of TightVNC v2.7.10 to handle keydown message
// - [DesktopWindow.cpp] bool DesktopWindow::onKey(...)
//

bool ClientConnection::onKey(WPARAM wParam, LPARAM lParam)
{
	unsigned short virtualKey = static_cast<unsigned short>(wParam);
	unsigned int additionalInfo = static_cast<unsigned int>(lParam);
	static const unsigned int DOWN_FLAG = 0x80000000;
	bool isDown = (additionalInfo & DOWN_FLAG) == 0;

	if (virtualKey == VK_CONTROL) {
		vnclog.Print(5, _T("[VNC] ClientConnection::onKey(): m_ctrlDown is true \n"));
		m_ctrlDown = isDown;
	}

	if (virtualKey == VK_MENU) {
		m_altDown = isDown;
	}

	vnclog.Print(5, _T("[VNC] ClientConnection::onKey(): virtualKey = %#4.4x \n"), (unsigned int)virtualKey);

	if(m_ctrlDown && virtualKey == 0x56)
	{
		if(isDown)
		{
			pasteClipboardText();
			return true;
		}
	}

	m_rfbKeySym->processKeyEvent(virtualKey, additionalInfo);

	return true;
}

//
// SendKeyEvent
//

inline void
	ClientConnection::SendKeyEvent(CARD32 key, bool down)
{
	rfbKeyEventMsg ke;
	ke.type = rfbKeyEvent;
	ke.down = down ? 1 : 0;
	ke.key = Swap32IfLE(key);
	WriteExact((char *)&ke, sz_rfbKeyEventMsg);
	vnclog.Print(7, _T("[VNC] ClientConnection::SendKeyEvent(): key = x%04x status = %s\n"), key, down ? _T("down") : _T("up"));
}

void ClientConnection::sendKeyboardEvent(bool downFlag, UINT32 key)
{
	SendKeyEvent((CARD32) key, downFlag);
}

void ClientConnection::sendVirtualKey(unsigned short usVirKeyCode)
{
	vnclog.Print(5, _T("[VNC] ClientConnection::sendVirtualKey(): key = x%04x\n"), usVirKeyCode);
	if(usVirKeyCode != 0xE9)
		m_rfbKeySym->processVirtualKey(usVirKeyCode);
	else
		SendKeyEvent(0xFFFE, true);
}

void ClientConnection::pasteClipboardText()
{
	if(IsTextClipboardFormat()&& m_bIsClintCutData)
	{
		vnclog.Print(6, _T("[VNC] ClientConnection::rightClickClipboard(): SendKeyEvent: 0xFFFC\n"));
		SendKeyEvent(0xFFFC, true);
	}
}

void ClientConnection::sendTouchDown(long lx, long ly)
{
	POINT point;
	point.x = (LONG)((float) (lx - m_iDeltaX) * m_fWidthRate);
	point.y = (LONG)((float) (ly - m_iDeltaY) * m_fHeightRate);

	// Check the position of Y is over the max height of screen size of phone
	if(point.y >= m_si.framebufferHeight)
	{
		vnclog.Print(5, "[VNC] ClientConnection::sendTouchDown(): point.y = %d (Touch Down) => Not send touch down event to phone !!\n", point.y);
		return;
	}

	if(point.x < 0) point.x = 0;
	if(point.y < 0) point.x = 0;

	//vnclog.Print(5, "[VNC] ClientConnection::sendTouchDown(): X = %d, Y = %d\n", point.x, point.y);
	SendPointerEvent(point.x, point.y, MOUSE_LDOWN);
}

void ClientConnection::sendTouchMove(long lx, long ly)
{
	Point point;
	point.x = (LONG)((float) (lx - m_iDeltaX) * m_fWidthRate);
	point.y = (LONG)((float) (ly - m_iDeltaY) * m_fHeightRate);

	// Check the position of Y is over the max height of screen size of phone
	if(point.y >= m_si.framebufferHeight)
	{
		vnclog.Print(5, "[VNC] ClientConnection::sendTouchMove(): point.y = %d (Touch Move) => Not send touch move event to phone !!\n", point.y);
		return;
	}

	if(point.x < 0) return;
	if(point.y < 0) return;

	if(!point.isEqualTo(&m_previousMousePos))
	{
		//vnclog.Print(5, "[VNC] ClientConnection::sendTouchMove(): X = %d, Y = %d\n", point.x, point.y);
		m_previousMousePos = point;
		SendPointerEvent(point.x, point.y, MOUSE_LDOWN);
	}
}

void ClientConnection::sendTouchUp(long lx, long ly)
{
	POINT point;
	point.x = (LONG)((float) (lx - m_iDeltaX) * m_fWidthRate);
	point.y = (LONG)((float) (ly - m_iDeltaY) * m_fHeightRate);

	if(point.x < 0) point.x = 0;
	if(point.y < 0) point.x = 0;

	//vnclog.Print(5, "[VNC] ClientConnection::sendTouchUp(): X = %d, Y = %d\n", point.x, point.y);
	SendPointerEvent(point.x, point.y, 0);	// 0 = Mouse Up
}

/// <summary>
/// Drops down notification list
/// </summary>
void ClientConnection::DropDownNotification()
{
	int iMove_X = 10;
	int iMove_Y = 0;
	if(this->m_bIsLandscapeMode)
	{
		// The screen is in Landscape mode
		iMove_X = (int)(m_si.framebufferHeight / 2);
	}
	else
	{
		// The screen is in Portrait mode
		iMove_X = (int)(m_si.framebufferWidth / 2);
	}

	// Simulate a series of touch control: Down -> Move right -> Up
	// Step(1): Touch down on the top of screen & move toward the bottom
	for(int i = 1; i <= 10; i++)
	{
		SendPointerEvent(iMove_X, iMove_Y, MOUSE_LDOWN);
		iMove_Y = iMove_Y + 50;
	}

	// Step(2): Touch up at the bottom of screen
	SendPointerEvent(iMove_X, iMove_Y, 0);	// 0 = Mouse Up
}

/// <summary>
/// Swipes view from left to right on phone screen
/// </summary>
void ClientConnection::SwipeLeftToRight()
{
	int iMove_X = 0;
	int iMove_Y = 0;
	if(this->m_bIsLandscapeMode)
	{
		// The screen is in Landscape mode
		iMove_Y = (int)(m_si.framebufferWidth / 2);
	}
	else
	{
		// The screen is in Portrait mode
		iMove_Y = (int)(m_si.framebufferHeight / 2);
	}

	// Simulate a series of touch control: Down -> Move right -> Up
	// Step(1): Touch down & move toward the right edge of screen
	for(int i = 1; i <= 10; i++)
	{
		iMove_X = iMove_X + 50;
		SendPointerEvent(iMove_X, iMove_Y, MOUSE_LDOWN);
	}

	// Step(2): Touch up at the right edge of screen
	iMove_X = iMove_X + 50;
	SendPointerEvent(iMove_X, iMove_Y, 0);	// 0 = Mouse Up
}

/// <summary>
/// Swipes view from right to left on phone screen
/// </summary>
void ClientConnection::SwipeRightToLeft()
{
	int iMove_X = 0;
	int iMove_Y = 0;
	if(this->m_bIsLandscapeMode)
	{
		// The screen is in Landscape mode
		iMove_X = m_si.framebufferHeight - 50;
		iMove_Y = (int)(m_si.framebufferWidth / 2);
	}
	else
	{
		// The screen is in Portrait mode
		iMove_X = m_si.framebufferWidth;
		iMove_Y = (int)(m_si.framebufferHeight / 2);
	}

	// Simulate a series of touch control: Down -> Move left -> Up
	// Step(1): Touch down & move toward the left edge of screen
	for(int i = 1; i <= 10; i++)
	{
		iMove_X = iMove_X - 50;
		SendPointerEvent(iMove_X, iMove_Y, MOUSE_LDOWN);
	}

	// Step(2): Touch up at the left edge of screen
	iMove_X = iMove_X - 50;
	SendPointerEvent(iMove_X, iMove_Y, 0);	// 0 = Mouse Up
}

//
// SendPointerEvent.
//

inline void
	ClientConnection::SendPointerEvent(int x, int y, int buttonMask)
{
	rfbPointerEventMsg pe;

	pe.type = rfbPointerEvent;
	pe.buttonMask = buttonMask;
	if (x < 0) x = 0;
	if (y < 0) y = 0;
	////SoftCursorMove(x, y);
	pe.x = Swap16IfLE(x);
	pe.y = Swap16IfLE(y);
	WriteExact((char *)&pe, sz_rfbPointerEventMsg);
}

//
// SendClientCutText
//

void ClientConnection::SendClientCutText(char *str, size_t len)
{
	rfbClientCutTextMsg cct;

	cct.type = rfbClientCutText;
	cct.length = Swap32IfLE(len);
	WriteExact((char *)&cct, sz_rfbClientCutTextMsg);
	WriteExact(str, len);
	vnclog.Print(6, _T("[VNC] ClientConnection::SendClientCutText(): Sent %d bytes of clipboard\n"), len);
}

//
// SendClientInputText
//

void ClientConnection::SendClientInputText(char *str, size_t len)
{
	rfbClientInputTextMsg cit;

	cit.type = rfbClientInputText;
	cit.length = Swap32IfLE(len);
	WriteExact((char *)&cit, sz_rfbClientInputTextMsg);
	WriteExact(str, len);
	vnclog.Print(8, _T("[VNC] ClientConnection::SendClientInputText(): Sent %d bytes of IME\n"), len);
}

#endif

//
// SendFileToServer
//

void ClientConnection::SendFileToServer(wchar_t* wsFilePath, int iFileId)
{
	//// First, check if the action is supported by the server.
	//// Upload was requested.
	//if ( !m_clientMsgCaps.IsEnabled(rfbFileUploadRequest) ||
	//	 !m_clientMsgCaps.IsEnabled(rfbFileUploadData) ) {
	//	//[ToDo] MessageBox(hwnd, "Sorry but the server does not support uploading files.", "Error", MB_OK | MB_ICONEXCLAMATION);
	//	return;
	//}

	// Now, try to upload/download.
	////if (m_pFileTransfer->m_ClientPath[0] == L'\0' || m_pFileTransfer->m_ServerPath[0] == L'\0') {
	////	//[ToDo] SetWindowText(_this->m_hwndFTStatus, "Cannot transfer files: illegal directory.");

	////	return;
	////}
	try
	{
		m_pFileTransfer->m_bTransferEnable = TRUE;
		m_pFileTransfer->m_bReportUploadCancel = TRUE;
		m_pFileTransfer->FileTransferUpload(wsFilePath, iFileId);
	}
	catch(VncException &e)
	{
		if(e.m_ErrType == EXP_SocketError)
			SendVncMessage(MT_ErrorMessage, FC_FileUpload, EC_SocketError, nullptr);
		else
			SendVncMessage(MT_ErrorMessage, FC_FileUpload, EC_OtherError, nullptr);
	}
}

// ********************************************************************
//  Methods after this point are generally called by the worker thread.
//  They finish the initialisation, then chiefly read data from the server.
// ********************************************************************

void* ClientConnection::run_undetached(void* arg) {
	vnclog.Print(5, _T("[VNC] ClientConnection::run_undetached(): Update-processing thread started\n"));

	m_threadStarted = true;
	CARD8 tmpRfbType = 0;
	try {
		// SendFullFramebufferUpdateRequest();
		// RealiseFullScreenMode(false);

		m_running = true;
		TCHAR _ReceviceDateTime[MAX_PATH];

		while (!m_bKillThread) {
			memset(_ReceviceDateTime, 0, MAX_PATH);
			// Look at the type of the message, but leave it in the buffer
			CARD8 msgType;
			{
				omni_mutex_lock l(m_readMutex);  // we need this if we're not using ReadExact
				int bytes = recv(m_sock, (char *) &msgType, 1, MSG_PEEK);

				if (bytes == 0)
				{
					m_pFileTransfer->CloseUndoneFileTransfers();
					vnclog.Print(0, _T("[ERROR] ClientConnection::run_undetached(): Connection closed !! (bytes = 0; err code = %d)\n"), WSAGetLastError() );
					throw WarningException(_T("Connection closed"));
				}

				if (bytes < 0)
				{
					m_pFileTransfer->CloseUndoneFileTransfers();
					vnclog.Print(0, _T("[ERROR] ClientConnection::run_undetached(): Socket error reading message (bytes < 0; err code = %d)\n"), WSAGetLastError() );
					throw WarningException("Error while waiting for server message");
				}
			}
			tmpRfbType = msgType;

			switch (msgType) {
			case rfbFramebufferUpdate:
				// ReadScreenUpdate();	// A] Server |
				break;
			case rfbSetColourMapEntries:
				ReadSetColourMapEntries();
				break;
			case rfbBell:
				// ReadBell();	// A] Server |
				break;
			case rfbServerCutText:
				vnclog.Print(5, _T("[VNC] ClientConnection::run_undetached(): Receive rfbServerCutText !!\n") );
				ReadServerCutText();
				break;
			case rfbFileListData:
				// [ToDoFun] m_pFileTransfer->ShowServerItems();
				break;
			case rfbFileDownloadData:
				m_pFileTransfer->FileTransferDownload();
				break;
			case rfbFileUploadCancel:
				vnclog.Print(5, _T("[VNC] ClientConnection::run_undetached(): Receive rfbFileUploadCancel !!\n") );
				m_pFileTransfer->ReadUploadCancel();
				break;
			case rfbFileDownloadFailed:
				vnclog.Print(5, _T("[VNC] ClientConnection::run_undetached(): Receive rfbFileDownloadFailed !!\n") );
				m_pFileTransfer->ReadDownloadFailed();
				break;

			default:
				vnclog.Print(0, _T("[ERROR] ClientConnection::run_undetached(): Unknown message type x%02x\n"), msgType );
				throw WarningException("Unhandled message type received!\n");
			}
		}
		vnclog.Print(5, _T("[VNC] ClientConnection::run_undetached(): Update-processing thread finishing\n") );
	}
	catch(VncException &e)
	{
		int iFuncCode = FC_FileDownload;

		if(tmpRfbType == rfbSetColourMapEntries)
			iFuncCode = FC_ColourMapEntries;
		else if(tmpRfbType == rfbServerCutText)
			iFuncCode = FC_ClipboardText;
		else if(tmpRfbType == rfbFileUploadCancel)
			iFuncCode = FC_FileUPloadCancel;

		if(e.m_ErrType == EXP_SocketError)
			SendVncMessage(MT_ErrorMessage, iFuncCode, EC_SocketError, nullptr);
		else if(e.m_ErrType == EXP_CustomError)
			SendVncMessage(MT_ErrorMessage, iFuncCode, e.m_iErrCode, nullptr);
		else
			SendVncMessage(MT_ErrorMessage, iFuncCode, EC_OtherError, nullptr);
	}
	catch (WarningException &e)
	{
		m_running = false;
		vnclog.Print(0, "[ERROR] ClientConnection::run_undetached(): m_running = false; WarningException = %s\n", e.m_info);
		//// PostMessage(m_hwnd1, WM_CLOSE, 0, 0);
		if (!m_bKillThread) {
			e.Report();
		}

		SendVncMessage(MT_ErrorMessage, FC_ServerRfbMsgHandler, EC_RFBMessageError, nullptr);
	} catch (QuietException &e)
	{
		m_running = false;
		vnclog.Print(1, "[ERROR] ClientConnection::run_undetached(): m_running = false; QuietException = %s\n", e.m_info);
		e.Report();
		//// PostMessage(m_hwnd1, WM_CLOSE, 0, 0);
		SendVncMessage(MT_ErrorMessage, FC_ServerRfbMsgHandler, EC_RFBMessageError, nullptr);
	}
	return this;
}

// The server has copied some text to the clipboard - put it
// in the local clipboard too.

void ClientConnection::ReadServerCutText()
{
	rfbServerCutTextMsg sctm;
	vnclog.Print(6, _T("[VNC] ClientConnection::ReadServerCutText(): Read remote clipboard change\n"));
	ReadExact((char *) &sctm, sz_rfbServerCutTextMsg);
	size_t len = Swap32IfLE(sctm.length);

	CheckBufferSize(len);
	if (len == 0) {
		m_netbuf[0] = '\0';
	} else {
		ReadString(m_netbuf, len);
	}

	UpdateLocalClipboard(m_netbuf, len);
}

void ClientConnection::ReadSetColourMapEntries()
{
	// Currently, we read and silently ignore SetColourMapEntries.

	rfbSetColourMapEntriesMsg msg;
	vnclog.Print(3, _T("Read server colour map entries (ignored)\n"));
	ReadExact((char *)&msg, sz_rfbSetColourMapEntriesMsg);
	int numEntries = Swap16IfLE(msg.nColours);

	if (numEntries > 0) {
		size_t nBytes = 6 * numEntries;
		CheckBufferSize(nBytes);
		ReadExact(m_netbuf, nBytes);
	}
}

// General utilities -------------------------------------------------

// Reads the number of bytes specified into the buffer given
void ClientConnection::ReadExact(char *inbuf, int wanted)
{
	if (m_sock == INVALID_SOCKET && m_bKillThread)
	{
		vnclog.Print(1, _T("[ERROR] ClientConnection::ReadExact(): Connection closed. m_sock = INVALID_SOCKET\n"));
		throw VncException("[ClientConnection::ReadExact][1]: Connection closed.", EXP_CustomError, EC_ConnectionClosed, true);
	}

	omni_mutex_lock l(m_readMutex);

	int offset = 0;
	vnclog.Print(10, _T("  reading %d bytes\n"), wanted);

	while (wanted > 0) {
		int bytes = recv(m_sock, inbuf+offset, wanted, 0);

		if (bytes == 0)
		{
			vnclog.Print(1, _T("[ERROR] ClientConnection::ReadExact(): Connection closed. recv bytes = 0\n"));
			throw VncException("[ClientConnection::ReadExact][2]: Connection closed.", EXP_CustomError, EC_ConnectionClosed, true);
		}

		if (bytes == SOCKET_ERROR) {
			int err = ::GetLastError();
			m_running = false;

			vnclog.Print(1, _T("[ERROR] ClientConnection::ReadExact(): Socket error while reading. recv bytes = SOCKET_ERROR (err code = %d)\n"), err);
			throw VncException("[ClientConnection::ReadExact][3]: Socket error while reading.", EXP_SocketError, err, true);
		}
		wanted -= bytes;
		offset += bytes;
	}
}

// Read the number of bytes and return them zero terminated in the buffer
inline void ClientConnection::ReadString(char *buf, int length)
{
	if (length > 0)
		ReadExact(buf, length);
	buf[length] = '\0';
	vnclog.Print(10, _T("Read a %d-byte string\n"), length);
}

// Sends the number of bytes specified from the buffer
inline void ClientConnection::WriteExact(char *buf, int bytes)
{
	if (bytes == 0 || m_sock == INVALID_SOCKET)
		return;

	omni_mutex_lock l(m_writeMutex);
	vnclog.Print(10, _T("  writing %d bytes\n"), bytes);

	int i = 0;
	int j;

	while (i < bytes) {
		j = send(m_sock, buf+i, bytes-i, 0);
		if (j == SOCKET_ERROR || j==0) {
			LPVOID lpMsgBuf;
			int err = WSAGetLastError();

			FormatMessage(
				FORMAT_MESSAGE_ALLOCATE_BUFFER |
				FORMAT_MESSAGE_FROM_SYSTEM |
				FORMAT_MESSAGE_IGNORE_INSERTS, NULL,
				err, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
				(LPTSTR) &lpMsgBuf, 0, NULL ); // Process any inserts in lpMsgBuf.

			vnclog.Print(1, _T("[ERROR] ClientConnection::WriteExact(): Socket error (send) %d: %s\n"), err, lpMsgBuf);
			LocalFree( lpMsgBuf );
			m_running = false;

			throw VncException("[ClientConnection::WriteExact] Socket error while writing.", EXP_SocketError, err, true);
		}
		i += j;
	}
}

// Read the string describing the reason for a connection failure.
// This function reads the data into m_netbuf, and returns that pointer
// as the beginning of the reason string.
char *ClientConnection::ReadFailureReason()
{
	CARD32 reasonLen;
	ReadExact((char *)&reasonLen, sizeof(reasonLen));
	reasonLen = Swap32IfLE(reasonLen);

	CheckBufferSize(reasonLen + 1);
	ReadString(m_netbuf, reasonLen);

	vnclog.Print(0, _T("[ERROR] ClientConnection::ReadFailureReason(): RFB connection failed, reason: %s\n"), m_netbuf);
	return m_netbuf;
}

// Makes sure netbuf is at least as big as the specified size.
// Note that netbuf itself may change as a result of this call.
// Throws an exception on failure.
void ClientConnection::CheckBufferSize(size_t bufsize)
{
	if (m_netbufsize > bufsize) return;

	// Don't try to allocate more than 2 gigabytes.
	if (bufsize >= 0x80000000) {
		vnclog.Print(1, _T("[ERROR] ClientConnection::CheckBufferSize(): Requested buffer size is too big (%u bytes)\n"), (unsigned int)bufsize);
		throw WarningException("Requested buffer size is too big.");
	}

	omni_mutex_lock l(m_bufferMutex);

	char *newbuf = new char[bufsize + 256];
	if (newbuf == NULL) {
		throw ErrorException("Insufficient memory to allocate network buffer.");
	}

	if (m_netbuf != NULL) {
		delete[] m_netbuf;
	}
	m_netbuf = newbuf;
	m_netbufsize = bufsize + 256;
	vnclog.Print(6, _T("[VNC] ClientConnection::CheckBufferSize(): Buffer size expanded to %u\n"), (unsigned int)m_netbufsize);
}

void ClientConnection::SetFileTransferUpdateCallback(VNCUploadFileProgessFun upload_callback, VNCDownloadFileProgessFun download_callback)
{
	if(this->m_pFileTransfer != nullptr)
	{
		this->m_pFileTransfer->FileUploadProgCallback = upload_callback;
		this->m_pFileTransfer->FileDownloadProgCallback = download_callback;
	}
}

void ClientConnection::SetCoordinateTransformationInfo(int iWidth, int iHeight, int iDeltaX, int iDeltaY, float fDpiRate)
{
	vnclog.Print(5, "[VNC] SetCoordinateTransformationInfo(): iWidth = %d; iHeight = %d; iX = %d; iY = %d; fDpiRate = %f\n", iWidth, iHeight, iDeltaX, iDeltaY, fDpiRate);
	if(iWidth <=0 || iHeight <=0 || fDpiRate <= 0.0f)
		return;

	m_iWinWidth = iWidth;
	m_iWinHeight = iHeight;
	m_iDeltaX = iDeltaX;
	m_iDeltaY = iDeltaY;
	m_fDPIRate = fDpiRate;

	if(iHeight >= iWidth)
	{
		// Portrait mode
		vnclog.Print(5, "[VNC] SetCoordinateTransformationInfo(): Portrait Mode !\n");
		m_bIsLandscapeMode = false;
		m_fWidthRate = m_si.framebufferWidth / (float) iWidth;
		m_fHeightRate = m_si.framebufferHeight / (float) iHeight;
	}
	else
	{
		// Landscape mode
		vnclog.Print(5, "[VNC] SetCoordinateTransformationInfo(): Landscape mode !\n");
		m_bIsLandscapeMode = true;
		m_fWidthRate = m_si.framebufferHeight / (float) iWidth;
		m_fHeightRate = m_si.framebufferWidth / (float) iHeight;
	}
}

void ClientConnection::SetDownloadFolderPath(wchar_t* folder_path)
{
	wmemchr(this->m_pFileTransfer->m_ClientPath, 0, rfbMAX_PATH);
	wcscpy(this->m_pFileTransfer->m_ClientPath, folder_path);
}

void ClientConnection::SendVncMessage(int msg_type, int fun_code, int err_code, char* msg)
{
	omni_mutex_lock l(m_callbackMutex);
	if(this->MessageCallback != nullptr)
	{
		MessageCallback(m_vnc_index, msg_type, fun_code, err_code, msg);
	}
}
